/*	This file is part of cfgd
**
**	Copyright 2002 Oliver Schroeder
**	
**	Permission to use,  copy, modify, and distribute this
**	software  and its  documentation  for any purpose and
**	without any fee is  hereby granted, provided that the
**	above  copyright notice  and  this  permission notice
**	appear in  all copies of the  software and derivative
**	works or modified versions thereof, and that both the
**	copyright notice and  this permission  and disclaimer
**	notice appear in supporting documentation.
**	
**	This software is provided "as is" WITHOUT ANY WARRANTY.
**	
**	Oliver Schroeder <post@o-schroeder.de>
*/

#include <stdlib.h>
#include "clist.h"

/*
**	create a new t_List
**
**	return: 0  - out of memory
**		!0 - ok, pointer to new t_List
*/
t_List* ListCreate ()
{
	t_List*	p_NewList;

	p_NewList = (t_List*) malloc (sizeof (t_List));
	if (p_NewList == 0)
	{
		return (0);
	}
	p_NewList->p_Current	= 0;
	p_NewList->p_Top	= 0;
	p_NewList->p_Bottom	= 0;
	pthread_mutex_init (& (p_NewList->MyMutex), 0);
	return (p_NewList);
}

/*
**	set pointer to the beginning of the list
**
**	return:	0 - first element does not exists
** 		1 - first element exists
*/
int ListSetStart
	(
	t_List* p_List 
	)
{
	if (p_List != 0)
	{
		pthread_mutex_lock (& (p_List->MyMutex));
		p_List->p_Current = p_List->p_Top;
		pthread_mutex_unlock (& (p_List->MyMutex));
		return (1);
	}
	return (0);
}

/*
**	set pointer to end of the list
**
**	return:	0 - last element does not exists
** 		1 - last element exists
*/
int ListSetEnd
	(
	t_List* p_List 
	)
{
	if (p_List != 0)
	{
		pthread_mutex_lock (& (p_List->MyMutex));
		p_List->p_Current = p_List->p_Bottom;
		pthread_mutex_unlock (& (p_List->MyMutex));
		return (1);
	}
	return (0);
}

/*
**	test if current element exists
**
**	return:	0 - current is 0
** 		1 - current exists (!NULL)
*/
void* ListCurrent
	( 
	t_List* p_List
	)
{
	if ((p_List != 0) && (p_List->p_Current))
	{
		return (p_List->p_Current);
	}
	return (0);
}

/*
**	return p_List->current and set it to the next element
**
**	return:  0 - there is no element
** 		!0 - the element
*/
void* ListNext
	(
	t_List* p_List
	)
{
	void*	vp_Content;

	if ((p_List != 0) && (p_List->p_Current != 0))
	{
		pthread_mutex_lock (& (p_List->MyMutex));
		vp_Content		= p_List->p_Current->vp_Content;
		p_List->p_Current	= p_List->p_Current->p_Next;
		pthread_mutex_unlock (& (p_List->MyMutex));
		return (vp_Content);
	}
	return (0);
}

/*
**	return p_List->current and set it to the previous element
**
**	return:	0 - there is no prev element
** 		1 - there is a prev element
*/
void* ListPrev 
	(
	t_List* p_List
	)
{
	void*	vp_Content;

	if ((p_List != 0) && (p_List->p_Current != 0))
	{
		pthread_mutex_lock (& (p_List->MyMutex));
		vp_Content		= p_List->p_Current->vp_Content;
		p_List->p_Current	= p_List->p_Current->p_Prev;
		pthread_mutex_unlock (& (p_List->MyMutex));
		return (vp_Content);
	}
	return (0);
}

/*
**	insert a new element in front of the list
**
**	return:	 0 - OK
**		!0 - Error
*/
int ListPushFront
	(
	t_List*	p_List,
	void*	vp_Content
	)
{
	t_ListNode*	p_NewNode;

	if (p_List == 0)
	{
		return (-1);
	}
	p_NewNode = (t_ListNode*) malloc (sizeof (t_ListNode));
	if (p_NewNode == 0)
	{
		return (-1);
	}
	p_NewNode->vp_Content	= vp_Content;
	p_NewNode->p_Prev	= 0;
	p_NewNode->p_Next	= p_List->p_Top;
	pthread_mutex_lock (& (p_List->MyMutex));
	if (p_List->p_Top != 0)
	{
		p_List->p_Top->p_Prev	= p_NewNode;
	}
	if (p_List->p_Bottom == 0)
	{
		p_List->p_Bottom = p_NewNode;
	}
	p_List->p_Top		= p_NewNode;
	pthread_mutex_unlock (& (p_List->MyMutex));
	return (0);
}

/*
**	insert a new element at the end of the list
**
**	return:	 0 - OK
**		!0 - Error
*/
int ListPushBack
	(
	t_List*	p_List,
	void*	vp_Content
	)
{
	t_ListNode*	p_NewNode;

	if (p_List == 0)
	{
		return (-1);
	}
	p_NewNode = (t_ListNode*) malloc (sizeof (t_ListNode));
	if (p_NewNode == 0)
	{
		return (-1);
	}
	p_NewNode->vp_Content	= vp_Content;
	p_NewNode->p_Prev	= p_List->p_Bottom;
	p_NewNode->p_Next	= 0;
	pthread_mutex_lock (& (p_List->MyMutex));
	if (p_List->p_Bottom != 0)
	{
		p_List->p_Bottom->p_Next= p_NewNode;
	}
	if (p_List->p_Top == 0)
	{
		p_List->p_Top = p_NewNode;
	}
	p_List->p_Bottom	= p_NewNode;
	pthread_mutex_unlock (& (p_List->MyMutex));
	return (0);
}

/*
**	return the first element and remove it from the list
**
**	return:	 0 - element or list does not exist
**		!0 - pointer to the element
*/
void* ListPopFront
	(
	t_List* p_List
	)
{
	t_ListNode*	p_Node;
	void*		vp_Content;

	if ((p_List == 0) || (p_List->p_Top == 0))
	{
		return (0);
	}
	pthread_mutex_lock (& (p_List->MyMutex));
	p_Node		= p_List->p_Top;
	p_List->p_Top	= p_List->p_Top->p_Next;
	if (p_Node->p_Prev != 0)
	{
		p_Node->p_Prev->p_Next = p_Node->p_Next;
	}
	if (p_Node->p_Next != 0)
	{
		p_Node->p_Next->p_Prev = p_Node->p_Prev;
	}
	pthread_mutex_unlock (& (p_List->MyMutex));
	vp_Content = p_Node->vp_Content;
	free (p_Node);
	return (vp_Content);
}

/*
**	return the last element and remove it from the list
**
**	return:	 0 - element or list does not exist
**		!0 - pointer to the element
*/
void* ListPopBack 
	(
	t_List* p_List
	)
{
	t_ListNode*	p_Node;
	void*		vp_Content;

	if ((p_List == 0) || (p_List->p_Bottom == 0))
	{
		return(0);
	}
	pthread_mutex_lock (& (p_List->MyMutex));
	p_Node			= p_List->p_Bottom;
	p_List->p_Bottom	= p_List->p_Bottom->p_Prev;
	if (p_Node->p_Prev != 0)
	{
		p_Node->p_Prev->p_Next = p_Node->p_Next;
	}
	if (p_Node->p_Next != 0)
	{
		p_Node->p_Next->p_Prev = p_Node->p_Prev;
	}
	pthread_mutex_unlock (& (p_List->MyMutex));
	vp_Content = p_Node->vp_Content;
	free (p_Node);
	return (vp_Content);
}

/*
**	return the first element of the list
**
**	return:	 0 - element or list does not exist
**		!0 - pointer to the element
*/
void* ListTop
	(
	t_List* p_List 
	)
{
	if ((p_List != 0) && (p_List->p_Top != 0))
	{
		return (p_List->p_Top->vp_Content);
	}
	return (0);
}

/*
**	return the last element of the list
**
**	return:	 0 - element or list does not exist
**		!0 - pointer to the element
*/
void* ListBottom
	(
	t_List* p_List
	)
{
	if ((p_List != 0) && (p_List->p_Bottom != 0))
	{
		return (p_List->p_Bottom->vp_Content);
	}
	return (0);
}

/*
**	Is the list empty?
**
**	return:	0 - no, there are elements
**		1 - yes, it's empty
*/
int ListEmpty
	(
	t_List* p_List
	)
{
	if ((p_List == 0) || (p_List->p_Top == 0))
	{
		return (1);
	}
	return (0);
}

/*
**
**	destroy the list
**
*/
void ListDestroy
	(
	t_List* p_List
	)
{
	t_ListNode*	p_Node;

	if (p_List == 0)
	{
		return;
	}
	pthread_mutex_lock (& (p_List->MyMutex));
	while (p_List->p_Top != 0)
	{
		p_Node		= p_List->p_Top;
		p_List->p_Top	= p_List->p_Top->p_Next;
		free (p_Node);
	};
	pthread_mutex_destroy (& (p_List->MyMutex));
	free (p_List);
	p_List = 0;
	pthread_mutex_unlock (& (p_List->MyMutex));
}

