/*
**	This file is part of cfgd
**
**      Copyright 2002 Oliver Schroeder
**
**      Permission to use, copy, modify, and distribute this
**      software and its documentation for any purpose and
**      without any fee is hereby granted, provided that the
**      above copyright notice and this permission notice
**      appear in all copies of the software and derivative
**      works or modified versions thereof, and that both the
**      copyright notice and this permission and disclaimer
**      notice appear in supporting documentation.
**
**      This software is provided "as is" WITHOUT ANY WARRANTY.
**
**      Oliver Schroeder <post@<o-schroeder.de>
**
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>
#include <expat.h>
#include "cconfig.h"
#include "cfgstack.h"

/*********************************************************************/
/**@name t_variable
 * 
 * structure for variables.
 * a user should not fiddle with this structure directly but use
 * the library functions instead. for that reason, the structure
 * is defined here and not in the header file.
 *
 * defined in 'cconfig.c'
 *
 * @see t_section
 * @see t_config
 *
 */
/*********************************************************************/
typedef struct st_variable
{
	/** name of the variable.
	 *
	 * can be anything, including whitespaces. but be careful
	 * with the seperator char!
	 *
	 * @see t_config
	 *
	 */
	char*	psz_Name;
	/** value of the variable
	 *
	 * can be anything you want.
	 *
	 */
	char*	psz_Value;
	/** an optional description for this variable
	 *
	 * this textfield should describe what this variable is meant
	 * for.
	 * 
	 */
	char*	psz_Description;
	/** an optional comment for this variable,
	 * 
	 * eg. why it is temporarily disabled etc.
	 *
	 */
	char*	psz_Comment;
	/** possible values for this variable.
	 *
	 * FIXME: not implemented yet
	 *
	 */
	t_CfgStack*	p_Predefs;
	/** the status of this variable
	 *
	 * eg. temp_disabled, root_only etc.
	 *
	 * FIXME: not implemented yet
	 *
	 */
	i_CFGSTATUS	n_Status;
	/** pointer to next variable
	 */
	struct  st_variable* p_Next;
	/** pointer to previous variable
	 */
	struct  st_variable* p_Prev;
} t_variable;

/*********************************************************************/
/**@name t_section
 *
 * structure for sections and subsections.
 * a user should not fiddle with this structure directly but use
 * the library functions instead. for that reason, the structure
 * is defined here and not in the header file.
 *
 * defined in 'cconfig.c'
 *
 * @see t_variable
 * @see t_config
 *
 */
/*********************************************************************/
typedef struct st_section
{
	/**
	 * name of this section
	 */
	char*	psz_Name;
	/**
	 * an optional description of this section
	 */
	char*	psz_Description;
	/**
	 * an optional comment of this section,
	 * eg. why it is disabled etc.
	 */
	char*	psz_Comment;
	/**
	 * the status of this section
	 * FIXME: not implemented yet
	 */
	i_CFGSTATUS	n_Status;
	/**
	 * variables in this section.
	 */
	struct	st_variable* p_Variables;
	/**
	 * subsections of this section
	 */
	struct	st_section* p_Subsections;
	/**
	 * parent-section of this section
	 */
	struct	st_section* p_Parent;
	/**
	 * pointer to next section
	 */
	struct	st_section* p_Next;
	/**
	 * pointer to previous section
	 */
	struct	st_section* p_Prev;
} t_section;

typedef struct stCfgStoreData
{
	struct st_variable*	p_CurrentVariable;
	struct st_section*	p_CurrentSection;
	struct st_variable*	p_FirstCurrentVariable;
	struct st_section*	p_FirstCurrentSection;
	char			c_Seperator;
} t_CfgStoreData;


/*********************************************************************/
/**@name t_config
 *
 * main structure of Cfg. hidden from the API-user, since the user
 * should do everything via the API-functions
 *
 * defined in 'cconfig.c'
 *
 * @see t_variable
 * @see t_section
 *
 */
/*********************************************************************/
struct st_config
{
	/**
	 * main variable list.
	 * these are not part of any section.
	 */
	struct st_variable*	p_Variables;
	/**
	 * sections of this Cfg.
	 */
	struct st_section*	p_Sections;
	/**
	 * pointer to the current variable.
	 */
	struct st_variable*	p_CurrentVariable;
	/**
	 * pointer the the current section.
	 */
	struct st_section*	p_CurrentSection;
	/**
	 * pointer the the first variable in the
	 * current variable list (which is possibly in
	 * any subsection)
	 */
	struct st_variable*	p_FirstCurrentVariable;
	/**
	 * pointer to the first section of
	 * current section list (which is
	 * possibly in any subsection)
	 */
	struct st_section*	p_FirstCurrentSection;
	/**
	 * seperator in strings containing section- and
	 * vaiable names (eg. for searching for a variable)
	 */
	char			c_Seperator;
	/**
	 * integer indicating an error of the last action
	 * done with this Cfg.
	 */
	i_CFGERROR		n_Error;
	t_CfgStack*		p_StoreStack;
	i_CFGINTERNALSFLAGS	n_Flags;
};

/**********************************************************************/
#include "cfghelpers.c"
#include "cfgparser.c"

/**********************************************************************/
/** API-Function
 *
 * create an empty t_config
 * create an empty configuration structur
 *
 * defined in 'cconfig.c'
 *
 * @return 0
 * 	if out of memory return 0
 * @return a pointer to the created t_config
 * 	if everyhting is allright.
 *
 */
/**********************************************************************/
t_config*
CfgCreate ()
{
	t_config* p_Cfg;

#if defined CCFG_DEBUG
	printf ("CfgCreate(): start\n");
#endif
	p_Cfg = (t_config*) malloc (sizeof (t_config));
	if (p_Cfg == 0)
	{
		return (0);
	}
	memset (p_Cfg, 0, sizeof (t_config));
	p_Cfg->p_Variables		= 0;
	p_Cfg->p_Sections		= 0;
	p_Cfg->p_CurrentSection		= p_Cfg->p_Sections;
	p_Cfg->p_FirstCurrentSection	= p_Cfg->p_Sections;
	p_Cfg->p_CurrentVariable	= p_Cfg->p_Variables;
	p_Cfg->p_FirstCurrentVariable	= p_Cfg->p_Variables;
	p_Cfg->n_Error			= CFGERROR_OK;
	p_Cfg->p_StoreStack		= 0;
	p_Cfg->c_Seperator		= '.';
	p_Cfg->n_Flags			= i_CFG_ISLOCAL|i_CFG_ISCONFIG;
	return (p_Cfg);
} /* CfgCreate() */

/**********************************************************************/
/** API-Function
 *
 * set the seperator
 *
 * (FIXME: write an understandable description for this :)
 *
 * defined in 'cconfig.c'
 *
 * @param p_Cfg
 * 	a valid pointer to a t_config
 * @param c_Seperator
 * 	the new seperator char
 * @precondition
 * 	p_Cfg must be valid or this function will
 * 	simply do nothing
 * @precondition
 * 	p_Cfg->n_Error must be CFGERROR_OK or this function will
 * 	simply do nothing
 * @return
 * 	nothing
 *
 */
/**********************************************************************/
void
CfgSetSeperator
	(
	t_config*	p_Cfg,
	char		c_Seperator
	)
{
#if defined CCFG_DEBUG
	printf ("CfgSetSeperator(%c): start\n", c_Seperator);
#endif
	if ((p_Cfg == 0) || (p_Cfg->n_Error != CFGERROR_OK))
	{
		return;
	}
	p_Cfg->c_Seperator = c_Seperator;
} /* CfgSetSeperator() */

/**********************************************************************/
/** API-Function
 *
 * return the error code of p_Cfg. reset p_Cfg->n_Error to 0
 *
 * defined in 'cconfig.c'
 *
 * @param p_Cfg
 * 	a valid pointer to a t_config
 * @return CFGERROR_NO_CREATE
 * 	if p_Cfg is non existant
 * @return
 * 	p_Cfg->n_Error else
 * @precondition
 * 	p_Cfg must be valid or this function will simply do nothing
 *
 */
/**********************************************************************/
int
CfgGetError
	(
	t_config*	p_Cfg
	)
{
	int n_Error;

#if defined CCFG_DEBUG
	printf ("CfgGetError(): start\n");
#endif
	if (p_Cfg == 0)
	{
		return (CFGERROR_NO_CREATE);
	}
	n_Error = p_Cfg->n_Error;
	p_Cfg->n_Error = CFGERROR_OK;
	return (n_Error);
} /* CfgGetError() */

/**********************************************************************/
/** API-Function
 *
 * read a config.
 *
 * defined in 'cconfig.c'
 *
 * FIXME: read from a file-descriptor or similar
 *
 * @return 0
 * 	could not open config
 * @return !=
 * 	everything ok
 *
 */
/**********************************************************************/
int
CfgReadConfig
	(
	t_config**	p_Cfg,
	char*		psz_Name
	)
{
	FILE*		p_CfgFile;
	

#if defined CCFG_DEBUG
	printf ("CfgReadConfig(): start!\n");
#endif
#if 0
	if (p_Cfg)
	{
		CfgFreeConfig (p_Cfg);
	}
#endif
	p_CfgFile = fopen (psz_Name, "r");
	if (!p_CfgFile)
	{
		printf ("error opening '%s'!\n", psz_Name);
		return (0);
	}
	i_CfgParseConfig (p_Cfg, p_CfgFile);
	fclose (p_CfgFile);
	return ((int) p_Cfg);
} /* CfgReadConfig() */

/**********************************************************************/
/**  API-Function
 *
 * delete the whole p_Cfg and free all used memory
 * frees all dynamically allocated memory consumed by a p_Cfg
 *
 * defined in 'cconfig.c'
 *
 * @param p_Cfg
 * 	a pointer the a valid t_config
 * @return
 * 	nothing
 * @precondition
 * 	p_Cfg must be valid or this function will simply do nothing
 *
 */
/**********************************************************************/
void
CfgFreeConfig
	(
	t_config* p_Cfg
	)
{
#if defined CCFG_DEBUG
	printf ("CfgFreeConfig(): start\n");
#endif
	if (p_Cfg == 0)
	{
		return;
	}
	i_CfgFreeVariables (p_Cfg->p_Variables);
	i_CfgFreeSection (p_Cfg->p_Sections);
	CfgStack_Destroy (p_Cfg->p_StoreStack);
	free (p_Cfg);
} /* CfgFreeConfig() */

/**********************************************************************/
/** API-Function
 *
 * print out p_Cfg in xml format (currently to stdout)
 *
 * defined in 'cconfig.c'
 *
 * @param fd
 * 	an allready opened file descriptor
 * @param p_Cfg
 * 	a pointer to a valid t_config
 * @precondition
 * 	p_Cfg must be valid or this function will simply do nothing
 * @precondition
 * 	p_Cfg->n_Error must be CFGERROR_OK or this
 * 	function will simply do nothing
 * @return
 * 	nothing
 *
 */
/**********************************************************************/
void
CfgDump
	(
	int		n_Fd,
	t_config*	p_Cfg
	)
{
	t_section*	p_Sections;
	t_variable*	p_Variables;
	FILE*		p_Target;

#if defined CCFG_DEBUG
	printf ("CfgDump(): start\n");
#endif
	if ((p_Cfg == 0) || (p_Cfg->n_Error != CFGERROR_OK))
	{
		return;
	}
	
	CfgSetStart (p_Cfg);
	p_Target	= fdopen (dup (n_Fd), "w");
	p_Variables	= p_Cfg->p_Variables;
	p_Sections	= p_Cfg->p_Sections;
#if 0
	fprintf (p_Target, "<?xml version=\"1.0\" ?>\n");
	fprintf (p_Target, "<config version=\"1.0\">\n");
	i_CfgWriteVariables (p_Target, p_Variables, 0);
	i_CfgWriteSection (p_Target, p_Sections, 0);
	fprintf (p_Target, "</config>\n");
#endif
	printf ("<?xml version=\"1.0\" ?>\n");
	printf ("<config version=\"1.0\">\n");
	i_CfgWriteVariables (p_Target, p_Variables, 0);
	i_CfgWriteSection (p_Target, p_Sections, 0);
	printf ("</config>\n");
	fclose (p_Target);
} /* CfgDump() */

/**********************************************************************/
/**  API-Function
 *
 * set pointers to the beginning of 'p_Cfg'.
 * t_config is like a tree. this function let all 'moving' pointers
 * point to the root of this tree.
 *
 * defined in 'cconfig.c'
 *
 * @param p_Cfg
 * 	a valid pointer to a t_config
 * @precondition
 * 	p_Cfg must be valid or this function will
 * 	simply do nothing
 * @precondition
 * 	p_Cfg->n_Error must be CFGERROR_OK or this
 * 	function will simply do nothing
 * @return
 * 	nothing
 *
 */
/**********************************************************************/
void
CfgSetStart
	(
	t_config*	p_Cfg
	)
{
	if ((p_Cfg == 0) || (p_Cfg->n_Error != CFGERROR_OK))
	{
		return;
	}
	p_Cfg->p_CurrentSection		= p_Cfg->p_Sections;
	p_Cfg->p_FirstCurrentSection	= p_Cfg->p_Sections;
	p_Cfg->p_CurrentVariable	= p_Cfg->p_Variables;
	p_Cfg->p_FirstCurrentVariable	= p_Cfg->p_Variables;
} /* CfgSetStart() */

/**********************************************************************/
/** API-Function
 *
 * t_config is like a tree.
 * this function let alle 'moving' pointers regarding sections to
 * the first section of the current branch. all 'moving' pointers
 * regarding variables of sections point to the first variable
 * of this section.
 *
 * defined in 'cconfig.c'
 *
 * @param p_Cfg
 * 	a valid pointer to a t_config
 * @precondition
 * 	p_Cfg must be valid or this function will
 * 	simply do nothing
 * @precondition
 * 	p_Cfg->n_Error must be CFGERROR_OK or this
 * 	function will simply do nothing
 * @return
 * 	nothing
 *
 */
/**********************************************************************/
void
CfgSetSecStart
	(
	t_config*	p_Cfg
	)
{
#if defined CCFG_DEBUG
	printf ("CfgSetSecStart(): start\n");
#endif
	if ((p_Cfg == 0) || (p_Cfg->n_Error != CFGERROR_OK))
	{
		return;
	}
	p_Cfg->p_CurrentSection = p_Cfg->p_FirstCurrentSection;
	if (p_Cfg->p_CurrentSection != 0)
	{
		p_Cfg->p_CurrentVariable =
			p_Cfg->p_CurrentSection->p_Variables;
		p_Cfg->p_FirstCurrentVariable =
			p_Cfg->p_CurrentSection->p_Variables;
	}
	else
	{
		p_Cfg->p_CurrentVariable = p_Cfg->p_Variables;
		p_Cfg->p_FirstCurrentVariable = p_Cfg->p_Variables;
	}
} /* CfgSetSecStart() */

/**********************************************************************/
/** API-Function
 *
 * set all 'moving' pointers regarding variables to the first
 * variable in the current branch of 'p_Cfg'
 *
 * defined in 'cconfig.c'
 *
 * @param p_Cfg
 * 	a valid pointer to a t_config
 * @return
 * 	nothing
 * @precondition
 *	p_Cfg must be valid or this function will simply do nothing
 * @precondition
 *      p_Cfg->n_Error must be CFGERROR_OK or this function will
 *	simply do nothing
 *
 */
/**********************************************************************/
void
CfgSetVarStart
	(
	t_config*	p_Cfg
	)
{
#if defined CCFG_DEBUG
	printf ("CfgSetVarStart start\n");
#endif
	if ((p_Cfg == 0) || (p_Cfg->n_Error != CFGERROR_OK))
	{
		return;
	}
	if (p_Cfg->p_CurrentSection != 0)
	{
		p_Cfg->p_CurrentVariable =
			p_Cfg->p_CurrentSection->p_Variables;
	}
	else
	{
		p_Cfg->p_CurrentVariable = p_Cfg->p_Variables;
	} /* else */
} /* CfgSetVarStart() */

/**********************************************************************/
/** API-Function
 *
 * find variable with 'psz_Name' in the current branch of 'p_Cfg'. this
 * function only inspects the current branch, it will neither walk
 * up nor down the tree.
 *
 * defined in 'cconfig.c'
 *
 * @param p_Cfg
 * 	a valid pointer to a t_config
 * @param psz_Name
 * 	the name of the variable to find
 * @precondition
 * 	p_Cfg must be valid or this function will simply do nothing
 * @precondition
 * 	p_Cfg->n_Error must be CFGERROR_OK or this
 * 	function will simply do nothing
 * @return 0
 * 	not found
 * @return 1
 * 	found
 *
 * @see CfgFindSection
 * @see CfgSetToVariable
 * @see CfgSetToSection
 * @see CfgFindVariable_global
 * @see CfgFindSection_global
 *
 */
/**********************************************************************/
int
CfgFindVariable
	(
	t_config*	p_Cfg,
	char*		psz_Name
	)
{
	t_variable*	p_CurrentVar;
	int		n_LenName;
	char*		n_Ret;

#if defined CCFG_DEBUG
	printf ("CfgFindVariable start\n");
	if (psz_Name != 0)
		printf ("\t# Name: '%s'\n", psz_Name);
	else    printf ("\t# no name given\n");
#endif
	if ((p_Cfg == 0) || (p_Cfg->n_Error != CFGERROR_OK))
	{
		return (0);
	}
	n_LenName = strlen (psz_Name);
	p_CurrentVar = p_Cfg->p_CurrentVariable;
	p_Cfg->p_CurrentVariable = p_Cfg->p_FirstCurrentVariable;
	n_Ret = (char*) 1;
	while ((n_Ret != 0) && (p_Cfg->p_CurrentVariable != 0))
	{
		if (p_Cfg->p_CurrentVariable->psz_Name == 0)
		{	/*
			 * FIXME: should not happen, but what to do
			 * 	   if it happens?
			 */
			return (0);
		}
		if (strncmp (p_Cfg->p_CurrentVariable->psz_Name,
			psz_Name, n_LenName) == 0)
		{
			return (1);
		}
		n_Ret = CfgNextVariable (p_Cfg);
	}
	p_Cfg->p_CurrentVariable = p_CurrentVar;
	return (0);
} /* CfgFindVariable() */

/**********************************************************************/
/** API-Function
 *
 * find section with 'psz_Name' in the current branch of 'p_Cfg'. this
 * function only inspects the current branch, it will neither walk
 * up nor down the tree.
 *
 * defined in 'cconfig.c'
 *
 * @param p_Cfg
 * 	a valid pointer to a t_config
 * @param psz_Name
 * 	the name of the section to find
 * @precondition
 * 	p_Cfg must be valid or this function will simply do nothing
 * @precondition
 * 	p_Cfg->n_Error must be CFGERROR_OK or this
 * 	function will simply do nothing
 * @return 0
 * 	not found
 * @return 1
 * 	found
 *
 * @see CfgFindVariable
 * @see CfgSetToVariable
 * @see CfgSetToSection
 * @see CfgFindVariable_global
 * @see CfgFindSection_global
 *
 */
/**********************************************************************/
int
CfgFindSection
	(
	t_config*	p_Cfg,
	char*		psz_Name
	)
{
	t_section*	p_CurrentSec;
	int		n_LenName;

#if defined CCFG_DEBUG
	printf ("CfgFindSection: start\n");
	if (psz_Name != 0)
		printf ("\t# Name: '%s'\n", psz_Name);
	else    printf ("\t# no name given\n");
#endif
	if ((p_Cfg == 0) || (p_Cfg->n_Error != CFGERROR_OK))
	{
		return (0);
	}
	n_LenName		= strlen (psz_Name);
	p_CurrentSec		= p_Cfg->p_CurrentSection;
	p_Cfg->p_CurrentSection	= p_Cfg->p_FirstCurrentSection;
	while (p_Cfg->p_CurrentSection != 0)
	{
		if (strncmp (p_Cfg->p_CurrentSection->psz_Name,
			psz_Name, n_LenName) == 0)
		{
			p_Cfg->p_CurrentVariable =
				p_Cfg->p_CurrentSection->p_Variables;
			p_Cfg->p_FirstCurrentVariable =
				p_Cfg->p_CurrentSection->p_Variables;
			return (1);
		}
		CfgNextSection (p_Cfg);
	}
	p_Cfg->p_CurrentSection = p_CurrentSec;
	return (0);
} /* CfgFindSection() */

/**********************************************************************/
/** API-Function
 *
 * add a new section to 'p_Cfg'. the new section is inserted at the 
 * current position of 'p_Cfg'. if you want to insert a section at
 * a specific position within 'p_Cfg' you should call CfgSetToSection
 * before.
 *
 * defined in 'cconfig.c'
 *
 * @param p_Cfg
 * 	pointer to a valid p_Cfg
 * @param psz_Name
 * 	of the new t_section
 * @param psz_Description
 * 	of the new t_section
 * @param psz_Comment
 * 	of the new t_section
 * @param n_Status
 * 	of the new t_section
 * @return CFGERROR_NO_CREATE
 * 	if p_Cfg does not exist.
 * @return CFGERROR_UNKNOWN
 * 	if p_Cfg->n_Error != CFGERROR_OK
 * @return CFGERROR_OUT_OF_MEMORY
 * 	if no memory could be allocated
 * @return CFGERROR_OK
 * 	everything is in order
 *
 * @see CfgSetToSection
 *
 */
/**********************************************************************/
int
CfgAddSection
	(
	t_config*	p_Cfg,
	char*		psz_Name,
	char*		psz_Description,
	char*		psz_Comment,
	i_CFGSTATUS	n_Status
	)
{
	t_section*	NewSection;

#if defined CCFG_DEBUG
	printf ("CfgAddSection: start\n");
	if (psz_Name != 0)
		printf ("\t# Name: '%s'\n", psz_Name);
	else    printf ("\t# no name given\n");
#endif
	if (p_Cfg == 0)
	{
		return (CFGERROR_NO_CREATE);
	}
	if (p_Cfg->n_Error != CFGERROR_OK)
	{
		return (CFGERROR_UNKNOWN);
	}
	NewSection = i_CfgCreateSection (p_Cfg, psz_Name,
		psz_Description, psz_Comment, n_Status);
	if (NewSection == 0)
	{
		p_Cfg->n_Error = CFGERROR_OUT_OF_MEMORY;
		return (CFGERROR_OUT_OF_MEMORY);
	}
	if (p_Cfg->p_Sections == 0)
	{
		NewSection->p_Prev		= 0;
		NewSection->p_Parent		= 0;
		p_Cfg->p_Sections		= NewSection;
		p_Cfg->p_CurrentSection		= p_Cfg->p_Sections;
		p_Cfg->p_FirstCurrentSection	= p_Cfg->p_Sections;
		return (CFGERROR_OK);
	}
	i_CfgEndOfSections (p_Cfg);
	NewSection->p_Prev	= p_Cfg->p_CurrentSection;
	NewSection->p_Parent	= p_Cfg->p_CurrentSection->p_Parent;
	p_Cfg->p_CurrentSection->p_Next	= NewSection;
	p_Cfg->p_CurrentSection	= p_Cfg->p_CurrentSection->p_Next;
#if defined CCFG_DEBUG
	i_CfgWriteSectionEx (NULL, p_Cfg->p_CurrentSection , 1);
#endif
	return (CFGERROR_OK);
} /* CfgAddSection() */

/**********************************************************************/
/** API-Function
 *
 * find a section named 'psz_Name' in 'p_Cfg'. this function will search
 * the whole tree of 'p_Cfg'.
 *
 * defined in 'cconfig.c'
 *
 * @param p_Cfg
 * 	a pointer to a valid t_config
 * @param psz_Name
 * 	of the section to find
 * @return 0
 * 	not found
 * @return 1
 * 	found
 * @precondition
 * 	p_Cfg must be valid or this function will simply do nothing
 * @see CfgFindVariable
 * @see CfgFindSection
 * @see CfgSetToVariable
 * @see CfgSetToSection
 * @see CfgFindVariable_global
 *
 */
/**********************************************************************/
int
CfgFindSection_global
	(
	t_config*	p_Cfg,
	char*		psz_Name
	)
{
#if defined CCFG_DEBUG
	printf ("CfgFindSection_global: start\n");
	if (psz_Name != 0)
		printf ("\t# Name: '%s'\n", psz_Name);
	else    printf ("\t# no name given\n");
#endif
	if (p_Cfg == 0)
	{
		return (CFGERROR_NO_CREATE);
	}
	CfgSetStart (p_Cfg);
	return (i_CfgFindSection (p_Cfg, psz_Name));
} /* CfgFindSection_global */

/**********************************************************************/
/** API-Function
 *
 * find variable 'psz_Name' in 'p_Cfg'
 *
 * defined in 'cconfig.c'
 *
 * @param p_Cfg
 * 	a pointer to a valid t_config
 * @param psz_Varname
 * 	of the variable including section-names
 * @precondition
 *	p_Cfg must be valid or this function will simply do nothing
 * @precondition
 *      p_Cfg->n_Error must be CFGERROR_OK or this function will
 *	simply do nothing
 * @return 
 * 	not found
 * @return 1
 * 	found
 *
 */
/**********************************************************************/
int
CfgSetToVariable
	(
	t_config*	p_Cfg,
	char*		psz_Varname
	)
{
#if defined CCFG_DEBUG
	printf ("CfgSetToVariable start\n");
	if (psz_Varname != 0)
		printf ("\t# Name: '%s'\n", psz_Varname);
	else    printf ("\t# no name given\n");
#endif
	if ((p_Cfg == 0) || (p_Cfg->n_Error != CFGERROR_OK))
	{
		return (CFGERROR_NO_CREATE);
	}
	CfgSetStart (p_Cfg);
	return (i_CfgSetToVariable (p_Cfg, psz_Varname));
} /* CfgSetToVariable */

/**********************************************************************/
/** API-Function
 *
 * find section 'pszSecname' in 'p_Cfg'
 *
 * defined in 'cconfig.c'
 *
 * @param p_Cfg
 * 	a pointer to a valid t_config
 * @param pszSecname
 * 	of the section including parent-p_Sections
 * @precondition
 *	p_Cfg must be valid or this function will simply do nothing
 * @precondition
 *      p_Cfg->n_Error must be CFGERROR_OK or this function will
 *	simply do nothing
 * @return 0
 * 	not found
 * @return 1
 * 	found
 *
 */
/**********************************************************************/
int
CfgSetToSection
	(
	t_config*	p_Cfg,
	char*		psz_Secname
	)
{
#if defined CCFG_DEBUG
	printf ("CfgSetToSection start\n");
	if (psz_Secname != 0)
		printf ("\t# Name: '%s'\n", psz_Secname);
	else    printf ("\t# no name given\n");

#endif
	if ((p_Cfg == 0) || (p_Cfg->n_Error != CFGERROR_OK))
	{
		return (CFGERROR_NO_CREATE);
	}
	CfgSetStart (p_Cfg);
	return (i_CfgSetToSection (p_Cfg, psz_Secname));
} /* CfgSetToVariable */

/**********************************************************************/
/** API-Function
 *
 * add a new SubSection to the current branch
 *
 * defined in 'cconfig.c'
 *
 * @param p_Cfg
 * 	a pointer to a valid t_config
 * @param psz_Name
 * 	name of the new SubSection
 * @param psz_Description
 * 	a description of the new SubSection
 * @param psz_Comment
 * 	a comment of the new SubSection
 * @param n_Status
 * 	a status of the new SebSection
 * @return CFGERROR_NO_CREATE
 * 	'p_Cfg' does not exist
 * @return CFGERROR_UNKNOWN
 * 	p_Cfg->n_Error != CFGERROR_OK or subsection already exists.
 * @return CFGERROR_NO_SECTION
 * 	current section does not exist
 */
/**********************************************************************/
int
CfgAddSubSection
	(
	t_config*	p_Cfg,
	char*		psz_Name,
	char*		psz_Description,
	char*		psz_Comment,
	i_CFGSTATUS	n_Status
	)
{
	t_section*	p_NewSection;

#if defined CCFG_DEBUG
	printf ("CfgAddSubSection: start\n");
	if (psz_Name != 0)
		printf ("\t# Name: '%s'\n", psz_Name);
	else    printf ("\t# no name given\n");
	printf ("CurrentSection:\n");
	i_CfgWriteSectionEx (NULL, p_Cfg->p_CurrentSection, 1);
#endif
	if (p_Cfg == 0)
	{
		return (CFGERROR_NO_CREATE);
	}
	if (p_Cfg->n_Error != CFGERROR_OK)
	{
		return (CFGERROR_UNKNOWN);
	}
	if (p_Cfg->p_CurrentSection == 0)
	{
		p_Cfg->n_Error = CFGERROR_NO_SECTION;
		return (CFGERROR_NO_SECTION);
	}
	if (p_Cfg->p_CurrentSection->p_Subsections != 0)
	{	/* subsection already exists */
		CfgNextSubSection (p_Cfg);
		i_CfgEndOfSections (p_Cfg);
		return (CfgAddSection (p_Cfg, psz_Name, psz_Description,
		  psz_Comment, n_Status));
	}
	p_NewSection = i_CfgCreateSection (p_Cfg, psz_Name,
		psz_Description, psz_Comment, n_Status);
	if (p_NewSection == 0)
	{
		p_Cfg->n_Error = CFGERROR_OUT_OF_MEMORY;
		return (CFGERROR_OUT_OF_MEMORY);
	}
	p_NewSection->p_Prev = 0;
	p_NewSection->p_Parent = p_Cfg->p_CurrentSection;
	p_Cfg->p_CurrentSection->p_Subsections = p_NewSection;
	p_Cfg->p_CurrentSection =
		p_Cfg->p_CurrentSection->p_Subsections;
	p_Cfg->p_FirstCurrentSection = p_Cfg->p_CurrentSection;
#if defined CCFG_DEBUG
	printf ("CurrentSection now:\n");
	i_CfgWriteSectionEx (NULL, p_Cfg->p_CurrentSection, 1);
#endif
	return (CFGERROR_OK);
} /* CfgAddSubSection() */

/**********************************************************************/
/** API-Function
 *
 * delete a section named 'pszSecname'
 *
 * defined in 'cconfig.c'
 *
 * @param p_Cfg
 * 	a pointer to a valid t_config
 * @param psz_SecName
 * 	name of the section to delete
 * @return 0
 * 	not specified yet, work to be done
 *
 * FIXME:		this function is horribly broken.
 * 			don't use it!
 */
/**********************************************************************/
int
CfgDelSection
	(
	t_config*	p_Cfg,
	char*		psz_SecName
	)
{
	int		n_Ret;
#if defined CCFG_DEBUG
	printf ("CfgDelSection(): not implemented yet!\n");
#endif
	if ((p_Cfg == 0) || (p_Cfg->n_Error != CFGERROR_OK))
	{
		return (0);
	}
	n_Ret = CfgFindSection (p_Cfg, psz_SecName);
	if (n_Ret == 0)
	{
		return (0);
	}
	i_CfgFreeSection (p_Cfg->p_CurrentSection);
	return (0);
} /* CfgDelSection() */

/**********************************************************************/
/** API-Function
 *
 * let p_Cfg->p_CurrentSection point to the next section in the list.
 *
 * defined in 'cconfig.c'
 *
 * @param p_Cfg
 * 	pointer to a valid t_config
 * @return 0
 * 	unable to set pointers to the next section
 * @return pointer to the name of next section
 * 	current pointers are moved to next section
 *
 */
/**********************************************************************/
char*
CfgNextSection
	(
	t_config*	p_Cfg
	)
{
#if defined CCFG_DEBUG
	printf ("CfgNextSection(): start\n");
	if (p_Cfg->p_CurrentSection != 0)
	{
	if (p_Cfg->p_CurrentSection->psz_Name != 0)
	{
		printf ("\tCurrent Section is '%s'\n",
		  p_Cfg->p_CurrentSection->psz_Name);
	}
	else	printf ("CurrentSection has no name\n");
	}
#endif
	if ((p_Cfg == 0) || (p_Cfg->n_Error != CFGERROR_OK))
	{
		return (0);
	}
	if (p_Cfg->p_CurrentSection == 0)
	{
		return (0);
	}
	if (p_Cfg->p_CurrentSection->p_Next != 0)
	{
		p_Cfg->p_CurrentSection =
			p_Cfg->p_CurrentSection->p_Next;
		p_Cfg->p_CurrentVariable =
			p_Cfg->p_CurrentSection->p_Variables;
		p_Cfg->p_FirstCurrentVariable =
			p_Cfg->p_CurrentSection->p_Variables;
#if defined CCFG_DEBUG
	if (p_Cfg->p_CurrentSection != 0)
	{
	if (p_Cfg->p_CurrentSection->psz_Name != 0)
	{
		printf ("\tCurrent Section is now '%s'\n",
		  p_Cfg->p_CurrentSection->psz_Name);
	}
	else	printf ("CurrentSection has no name\n");
	}
#endif
		return (p_Cfg->p_CurrentSection->psz_Name);
	}
	else
	{
		p_Cfg->p_CurrentSection =
			p_Cfg->p_CurrentSection->p_Next;
	}
	return (0);
} /* CfgNextSection() */

/**********************************************************************/
/** API-Function
 *
 * let p_Cfg->p_CurrentSection point to next sub-section in the list.
 *
 * defined in 'cconfig.c'
 *
 * @param p_Cfg
 * 	pointer to a valid t_config
 * @return 0
 * 	unable to set pointers to the next subsection
 * @return pointer to the name of next subsection
 * 	current pointers are moved to next subsection
 *
 */
/**********************************************************************/
char*
CfgNextSubSection 
	(
	t_config*	p_Cfg
	)
{
#if defined CCFG_DEBUG
	printf ("CfgNextSubSection(): start\n");
	if (p_Cfg->p_CurrentSection != 0)
	{
	if (p_Cfg->p_CurrentSection->psz_Name != 0)
	{
		printf ("\tCurrent Section is '%s'\n",
		  p_Cfg->p_CurrentSection->psz_Name);
	}
	else	printf ("CurrentSection has no name\n");
	}
#endif
	if ((p_Cfg == 0) || (p_Cfg->n_Error != CFGERROR_OK))
	{
		return (0);
	}
	if (p_Cfg->p_CurrentSection == 0)
	{
		return (0);
	}
	if (p_Cfg->p_CurrentSection->p_Subsections != 0)
	{
		p_Cfg->p_CurrentSection =
			p_Cfg->p_CurrentSection->p_Subsections;
		p_Cfg->p_FirstCurrentSection =
			p_Cfg->p_CurrentSection;
		p_Cfg->p_CurrentVariable =
			p_Cfg->p_CurrentSection->p_Variables;
		p_Cfg->p_FirstCurrentVariable =
			p_Cfg->p_CurrentSection->p_Variables;
#if defined CCFG_DEBUG
	if (p_Cfg->p_CurrentSection != 0)
	{
	if (p_Cfg->p_CurrentSection->psz_Name != 0)
	{
		printf ("\tCurrent Section is '%s'\n",
		  p_Cfg->p_CurrentSection->psz_Name);
	}
	else	printf ("CurrentSection has no name\n");
	}
#endif
		return (p_Cfg->p_CurrentSection->psz_Name);
	}
	return (0);
} /* CfgNextSubSection() */

/**********************************************************************/
/** API-Function
 *
 * let p_Cfg->p_CurrentSection point to
 * p_Cfg->p_CurrentSection->p_Parent
 *
 * defined in 'cconfig.c'
 *
 */
/**********************************************************************/
char*
CfgParentSection
	(
	t_config*	p_Cfg
	)
{
#if defined CCFG_DEBUG
	printf ("CfgParentSection(): start\n");
#endif
	if ((p_Cfg == 0) || (p_Cfg->n_Error != CFGERROR_OK))
	{
		return (0);
	}
	if (p_Cfg->p_CurrentSection == 0)
	{
		return (0);
	}
	if (p_Cfg->p_CurrentSection->p_Parent != 0)
	{
		p_Cfg->p_CurrentSection =
			p_Cfg->p_CurrentSection->p_Parent;
		p_Cfg->p_FirstCurrentSection =
			p_Cfg->p_CurrentSection;
		if (p_Cfg->p_FirstCurrentSection != 0)
		{
			while (p_Cfg->p_FirstCurrentSection->p_Prev != 0)
			{
				p_Cfg->p_FirstCurrentSection =
					p_Cfg->p_FirstCurrentSection->p_Prev;
			}
		}
		p_Cfg->p_CurrentVariable =
			p_Cfg->p_CurrentSection->p_Variables;
		p_Cfg->p_FirstCurrentVariable =
			p_Cfg->p_CurrentSection->p_Variables;
		if (p_Cfg->p_FirstCurrentVariable != 0)
		{
			while (p_Cfg->p_FirstCurrentVariable->p_Prev != 0)
			{
				p_Cfg->p_FirstCurrentVariable =
					p_Cfg->p_FirstCurrentVariable->p_Prev;
			}
		}
		return (p_Cfg->p_CurrentSection->psz_Name);
	}
	return (0);
} /* CfgParentSection() */

/**********************************************************************/
/** API-Function
 * 
 * let p_Cfg->p_CurrentVariable point to
 * p_Cfg->p_CurrentVariable->p_Next
 *
 * defined in 'cconfig.c'
 *
 */
/**********************************************************************/
char*
CfgNextVariable
	(
	t_config*	p_Cfg
	)
{
#if defined CCFG_DEBUG
	printf ("CfgNextVariable(): start\n");
	if (p_Cfg->p_CurrentVariable != 0)
	{
	if (p_Cfg->p_CurrentVariable->psz_Name != 0)
	{
		printf ("\tCurrent Variable is '%s'\n",
		  p_Cfg->p_CurrentVariable->psz_Name);
	}
	else	printf ("CurrentVariable has no name\n");
	}
#endif
	if ((p_Cfg == 0) || (p_Cfg->n_Error != CFGERROR_OK)
	|| (p_Cfg->p_CurrentVariable == 0))
	{
		return (0);
	}
	if (p_Cfg->p_CurrentVariable->p_Next != 0)
	{
		p_Cfg->p_CurrentVariable =
			p_Cfg->p_CurrentVariable->p_Next;
		return (p_Cfg->p_CurrentVariable->psz_Name);
	}
	return (0);
} /* CfgNextVariable() */

/**********************************************************************/
/** API-Function
 * 
 * let p_Cfg->p_CurrentVariable point to
 * p_Cfg->p_CurrentVariable->p_Prev
 *
 * defined in 'cconfig.c'
 *
 */
/**********************************************************************/
char*
CfgPrevVariable
	(
	t_config*	p_Cfg
	)
{
#if defined CCFG_DEBUG
	printf ("CfgPrevVariable(): start\n");
#endif
	if ((p_Cfg == 0) || (p_Cfg->n_Error != CFGERROR_OK)
	|| (p_Cfg->p_CurrentVariable == 0))
	{
		return (0);
	}
	if (p_Cfg->p_CurrentVariable->p_Prev != 0)
	{
		p_Cfg->p_CurrentVariable =
			p_Cfg->p_CurrentVariable->p_Prev;
		return (p_Cfg->p_CurrentVariable->psz_Name);
	}
	return (0);
} /* CfgPrevVariable() */

/**********************************************************************/
/** API-Function
 *
 * add a new variable to the list
 *
 * defined in 'cconfig.c'
 *
 */
/**********************************************************************/
int
CfgAddVariable
	(
	t_config*	p_Cfg,
	char*		psz_Name,
	char*		psz_Value,
	char*		psz_Description,
	char*		psz_Comment,
	i_CFGSTATUS	n_Status
	)
{
	t_variable*	p_NewVariable;

#if defined CCFG_DEBUG
	printf ("CfgAddVariable: start\n");
	if (psz_Name != 0)
		printf ("\t# Name: '%s'\n", psz_Name);
	else    printf ("\t# no name given\n");
#endif
	if ((p_Cfg == 0) || (p_Cfg->n_Error != CFGERROR_OK))
	{
		return (CFGERROR_NO_CREATE);
	}
	p_NewVariable = (t_variable*) malloc (sizeof (t_variable));
	if (p_NewVariable == 0)
	{
		p_Cfg->n_Error = CFGERROR_OUT_OF_MEMORY;
		return (CFGERROR_OUT_OF_MEMORY);
	} /* if */
	memset (p_NewVariable, 0, sizeof (t_variable));
	/** FIXME: handle return code of i_CfgStrCpy */
	i_CfgStrCpy (&(p_NewVariable->psz_Name), psz_Name);
	i_CfgStrCpy (&(p_NewVariable->psz_Value), psz_Value);
	i_CfgStrCpy (&(p_NewVariable->psz_Description),
		psz_Description);
	i_CfgStrCpy (&(p_NewVariable->psz_Comment), psz_Comment);
	p_NewVariable->n_Status = n_Status;
	p_NewVariable->p_Predefs = 0;
	if (p_Cfg->p_CurrentSection == 0)
	{
		if (p_Cfg->p_CurrentVariable == 0)
		{
			p_NewVariable->p_Prev = 0;
			p_Cfg->p_Variables = p_NewVariable;
			p_Cfg->p_CurrentVariable = p_Cfg->p_Variables;
			p_Cfg->p_FirstCurrentVariable =
				p_Cfg->p_Variables;
			return (CFGERROR_OK);
		}
		i_CfgEndOfVariables (p_Cfg);
		p_NewVariable->p_Prev = p_Cfg->p_CurrentVariable;
		p_Cfg->p_CurrentVariable->p_Next = p_NewVariable;
		p_Cfg->p_CurrentVariable =
			p_Cfg->p_CurrentVariable->p_Next;
	}
	else
	{
		p_Cfg->p_CurrentVariable =
			p_Cfg->p_CurrentSection->p_Variables;
		if (p_Cfg->p_CurrentVariable == 0)
		{
			p_NewVariable->p_Prev = 0;
			p_Cfg->p_CurrentSection->p_Variables =
				p_NewVariable;
			p_Cfg->p_CurrentVariable  =
				p_Cfg->p_CurrentSection->p_Variables;
			p_Cfg->p_FirstCurrentVariable =
				p_Cfg->p_CurrentSection->p_Variables;
			return (CFGERROR_OK);
		}
		p_Cfg->p_FirstCurrentVariable =
			p_Cfg->p_CurrentSection->p_Variables;
		i_CfgEndOfVariables (p_Cfg);
		p_NewVariable->p_Prev =
			p_Cfg->p_CurrentVariable;
		p_Cfg->p_CurrentVariable->p_Next =
			p_NewVariable;
		p_Cfg->p_CurrentVariable =
			p_Cfg->p_CurrentVariable->p_Next;
	} /* else */
	return (CFGERROR_OK);
} /* CfgAddVariable() */

/**********************************************************************/
/** API-Function
 *
 * return the name of the current section
 *
 * defined in 'cconfig.c'
 *
 */
/**********************************************************************/
char*
CfgGetSectionName
	(
	t_config*	p_Cfg
	)
{
#if defined CCFG_DEBUG
	printf ("CfgGetSectionName() start\n");
#endif
	if (!i_CfgCurrentSectionExists (p_Cfg))
	{
		return (0);
	}
	if (IsReadable (p_Cfg, p_Cfg->p_CurrentSection->n_Status) == 0)
	{
		return (0);
	}
	return (p_Cfg->p_CurrentSection->psz_Name);
} /* CfgGetSectionName() */

/**********************************************************************/
/** API-Function
 *
 * return the desccription of the current section
 *
 * defined in 'cconfig.c'
 *
 */
/**********************************************************************/
char*
CfgGetSectionDescription 
	(
	t_config*	p_Cfg
	)
{
#if defined CCFG_DEBUG
	printf ("CfgGetSectionDescription(): start\n");
#endif
	if (!i_CfgCurrentSectionExists (p_Cfg))
	{
		return (0);
	}
	return (p_Cfg->p_CurrentSection->psz_Description);
} /* CfgGetSectionDescription() */

/**********************************************************************/
/** API-Function
 *
 * return the comment of the current section
 *
 * defined in 'cconfig.c'
 *
 */
/**********************************************************************/
char*
CfgGetSectionComment
	(
	t_config*	p_Cfg
	)
{
#if defined CCFG_DEBUG
	printf ("CfgGetSectionComment(): start\n");
#endif
	if (!i_CfgCurrentSectionExists (p_Cfg))
	{
		return (0);
	}
	return (p_Cfg->p_CurrentSection->psz_Comment);
} /* CfgGetSectionComment() */

/**********************************************************************/
/** API-Function
 *
 * return the status of the current section
 *
 * defined in 'cconfig.c'
 *
 */
/**********************************************************************/
int
CfgGetSection_Status
	(
	t_config*	p_Cfg
	)
{
#if defined CCFG_DEBUG
	printf ("CfgGetSection_Status(): start\n");
#endif
	if (!i_CfgCurrentSectionExists (p_Cfg))
	{
		return (CFGERROR_NO_SECTION);
	}
	return (p_Cfg->p_CurrentSection->n_Status);
} /* CfgGetSection_Status() */

/**********************************************************************/
/** API-Function
 *
 * return the number of sections of the current context
 *
 * defined in 'cconfig.c'
 *
 */
/**********************************************************************/
int
CfgGetSectionCount
	(
	t_config*	p_Cfg
	)
{
	t_section*	p_Sec;
	int		n_Count;

#if defined CCFG_DEBUG
	printf ("CfgGetSectionCount(): start\n");
#endif
	if ((p_Cfg == 0) || (p_Cfg->n_Error != CFGERROR_OK))
	{
		return (0);
	}
	n_Count = 0;
	p_Sec = p_Cfg->p_FirstCurrentSection;
	while (p_Sec != 0)
	{
		n_Count++;
		p_Sec = p_Sec->p_Next;
	}
	return (n_Count);
} /* CfgGetSectionCount() */

/**********************************************************************/
/** API-Function
 *
 * set the name of p_Cfg->p_CurrentSection
 *
 * defined in 'cconfig.c'
 *
 */
/**********************************************************************/
int
CfgSetSectionName
	(
	t_config*	p_Cfg,
	char*		psz_Name
	)
{
#if defined CCFG_DEBUG
	printf ("CfgSetSectionName: start\n");
	if (psz_Name != 0)
		printf ("\t# Name: '%s'\n", psz_Name);
	else    printf ("\t# no name given\n");
#endif
	if (!i_CfgCurrentSectionExists (p_Cfg))
	{
		return (CFGERROR_NO_SECTION);
	}
	if (p_Cfg->p_CurrentSection->psz_Name != 0)
	{
		free (p_Cfg->p_CurrentSection->psz_Name);
		p_Cfg->p_CurrentSection->psz_Name = 0;
	}
	return (i_CfgStrCpy (&(p_Cfg->p_CurrentSection->psz_Name),
		psz_Name));
} /* CfgSetSectionName() */

/**********************************************************************/
/** API-Function
 *
 * set the description of p_Cfg->p_CurrentSection
 *
 * defined in 'cconfig.c'
 *
 */
/**********************************************************************/
int
CfgSetSectionDescription
	(
	t_config*	p_Cfg,
	char*		psz_Description
	)
{
#if defined CCFG_DEBUG
	printf ("CfgSetSectionDescription: start\n");
	if (psz_Description != 0)
		printf ("\t# Name: '%s'\n", psz_Description);
	else    printf ("\t# no name given\n");
#endif
	if (!i_CfgCurrentSectionExists (p_Cfg))
	{
		return (CFGERROR_NO_SECTION);
	}
	if (p_Cfg->p_CurrentSection->psz_Description != 0)
	{
		free (p_Cfg->p_CurrentSection->psz_Description);
		p_Cfg->p_CurrentSection->psz_Description = 0;
	}
	return (i_CfgStrCpy(&(p_Cfg->p_CurrentSection->psz_Description),
		psz_Description));
} /* CfgSetSectionDescription() */

/**********************************************************************/
/** API-Function
 *
 * set the comment of p_Cfg->p_CurrentSection
 *
 * defined in 'cconfig.c'
 *
 */
/**********************************************************************/
int
CfgSetSectionComment
	(
	t_config*	p_Cfg,
	char*		psz_Comment
	)
{
#if defined CCFG_DEBUG
	printf ("CfgSetSectionComment: start\n");
	if (psz_Comment != 0)
		printf ("\t# Name: '%s'\n", psz_Comment);
	else    printf ("\t# no name given\n");
#endif
	if (!i_CfgCurrentSectionExists (p_Cfg))
	{
		return (CFGERROR_NO_SECTION);
	}
	if (p_Cfg->p_CurrentSection->psz_Comment != 0)
	{
		free (p_Cfg->p_CurrentSection->psz_Comment);
		p_Cfg->p_CurrentSection->psz_Comment = 0;
	}
	return (i_CfgStrCpy (&(p_Cfg->p_CurrentSection->psz_Comment),
		psz_Comment));
} /* CfgSetSectionComment() */

/**********************************************************************/
/** API-Function
 *
 * set the status of p_Cfg->p_CurrentSection
 *
 * defined in 'cconfig.c'
 *
 */
/**********************************************************************/
int
CfgSetSectionStatus
	(
	t_config*	p_Cfg,
	i_CFGSTATUS	n_Status
	)
{
#if defined CCFG_DEBUG
	printf ("CfgSetSectionStatus('%d'): start\n", n_Status);
#endif
	if (!i_CfgCurrentSectionExists (p_Cfg))
	{
		return (CFGERROR_NO_SECTION);
	}
	p_Cfg->p_CurrentSection->n_Status = n_Status;
	return (CFGERROR_OK);
} /* CfgSetSectionStatus() */

/**********************************************************************/
/** API-Function
 *
 * find a variable named 'psz_Varname' and return its value
 *
 * defined in 'cconfig.c'
 *
 */
/**********************************************************************/
char*
CfgGetVarValueOf
	(
	t_config*	p_Cfg,
	char*		psz_Varname
	)
{
#if defined CCFG_DEBUG
	printf ("CfgGetVarValueOf: start\n");
	if (psz_Varname != 0)
		printf ("\t# Name: '%s'\n", psz_Varname);
	else    printf ("\t# no name given\n");
#endif
	if ((p_Cfg == 0) || (p_Cfg->n_Error != CFGERROR_OK))
	{
		return (0);
	}
	if (CfgSetToVariable (p_Cfg, psz_Varname) == 0)
	{
		return (0);
	}
	if (IsReadable (p_Cfg, p_Cfg->p_CurrentSection->n_Status) == 0)
	{
		return (0);
	}
	return (p_Cfg->p_CurrentVariable->psz_Value);
} /* CfgGetVarValueOf() */

/**********************************************************************/
/** API-Function
 *
 * find a variable named 'psz_Varname' and return its description
 *
 * defined in 'cconfig.c'
 *
 */
/*********************************************************************/
char*
CfgGetVarDescriptionOf
	(
	t_config*	p_Cfg,
	char*		psz_Varname
	)
{
#if defined CCFG_DEBUG
	printf ("CfgGetVarDescriptionOf: start\n");
	if (psz_Varname != 0)
		printf ("\t# Name: '%s'\n", psz_Varname);
	else    printf ("\t# no name given\n");
#endif
	if ((p_Cfg == 0) || (p_Cfg->n_Error != CFGERROR_OK))
	{
		return (0);
	}
	if (CfgSetToVariable (p_Cfg, psz_Varname) == 0)
	{
		return (0);
	}
	return (p_Cfg->p_CurrentVariable->psz_Description);
} /* CfgGetVarDescriptionOf() */

/**********************************************************************/
/** API-Function
 *
 * find a variable named 'psz_Varname' and return its comment.
 *
 * defined in 'cconfig.c'
 *
 */
/**********************************************************************/
char*
CfgGetVarCommentOf
	(
	t_config*	p_Cfg,
	char*		psz_Varname
	)
{
#if defined CCFG_DEBUG
	printf ("CfgGetVarCommentOf: start\n");
	if (psz_Varname != 0)
		printf ("\t# Name: '%s'\n", psz_Varname);
	else    printf ("\t# no name given\n");
#endif
	if ((p_Cfg == 0) || (p_Cfg->n_Error != CFGERROR_OK))
	{
		return (0);
	}
	if (CfgSetToVariable (p_Cfg, psz_Varname) == 0)
	{
		return (0);
	}
	return (p_Cfg->p_CurrentVariable->psz_Comment);
} /* CfgGetVarDescriptionOf() */

/**********************************************************************/
/** API-Function
 *
 * find a variable named 'psz_Varname' and return its status
 *
 * defined in 'cconfig.c'
 *
 */
/**********************************************************************/
int
CfgGetVarStatusOf 
	(
	t_config*	p_Cfg,
	char*		psz_Varname
	)
{
#if defined CCFG_DEBUG
	printf ("CfgGetVarStatusOf: start\n");
	if (psz_Varname != 0)
		printf ("\t# Name: '%s'\n", psz_Varname);
	else    printf ("\t# no name given\n");
#endif
	if ((p_Cfg == 0) || (p_Cfg->n_Error != CFGERROR_OK))
	{
		return (0);
	}
	if (CfgSetToVariable (p_Cfg, psz_Varname) == 0)
	{
		return (0);
	}
	return (p_Cfg->p_CurrentVariable->n_Status);
} /* CfgGetVarStatusOf() */

/**********************************************************************/
/** API-Function
 *
 * find a variable named 'psz_Varname' and return its predefs.
 *
 * defined in 'cconfig.c'
 *
 */
/**********************************************************************/
char*
CfgGetVarPredefOf
	(
	t_config*	p_Cfg,
	char*		psz_Varname
	)
{
#if defined CCFG_DEBUG
	printf ("CfgGetVarPredefOf: start\n");
	if (psz_Varname != 0)
		printf ("\t# Name: '%s'\n", psz_Varname);
	else    printf ("\t# no name given\n");
#endif
	if ((p_Cfg == 0) || (p_Cfg->n_Error != CFGERROR_OK))
	{
		return (0);
	}
	if (CfgSetToVariable (p_Cfg, psz_Varname) == 0)
	{
		return (0);
	}
	CfgStack_SetStart (p_Cfg->p_CurrentVariable->p_Predefs);
	return (CfgStack_Next (p_Cfg->p_CurrentVariable->p_Predefs));
} /* CfgGetVarPredefOf() */

/**********************************************************************/
/** API-Function
 *
 * find a variable named 'psz_Varname' and set a new value for it
 *
 * defined in 'cconfig.c'
 * FIXME: return values
 *
 */
/**********************************************************************/
int
CfgSetVarValueOf
	(
	t_config*	p_Cfg,
	char*		psz_Varname,
	char*		psz_Value
	)
{
#if defined CCFG_DEBUG
	printf ("CfgSetVarValueOf: start\n");
	if (psz_Varname != 0)
		        printf ("\t# Name: '%s'\n", psz_Varname);
	else    printf ("\t# no name given\n");
	if (psz_Value != 0)
		        printf ("\t# Value: '%s'\n", psz_Value);
	else    printf ("\t# no value given\n");
#endif
	if ((p_Cfg == 0) || (p_Cfg->n_Error != CFGERROR_OK))
	{
		return (0);
	}
	/*
	 * FIXME: CfgSetToVariable checks (p_Cfg == 0) etc., too
	 */
	if (CfgSetToVariable (p_Cfg, psz_Varname) == 0)
	{
		return (0);
	}
	if (p_Cfg->p_CurrentVariable->psz_Value != 0)
	{
		free (p_Cfg->p_CurrentVariable->psz_Value);
		p_Cfg->p_CurrentVariable->psz_Value = 0;
	}
	i_CfgStrCpy (&(p_Cfg->p_CurrentVariable->psz_Value), psz_Value);
	return (1);
} /* CfgSetVarValueOf() */

/**********************************************************************/
/** API-Function
 *
 * find a variable named 'psz_Varname' and set a new description for it
 *
 * defined in 'cconfig.c'
 *
 */
/**********************************************************************/
int
CfgSetVarDescriptionOf
	(
	t_config*	p_Cfg,
	char*		psz_Varname,
	char*		psz_Description
	)
{
#if defined CCFG_DEBUG
	printf ("CfgSetVarDescriptionOf: start\n");
	if (psz_Varname != 0)
		printf ("\t# Name: '%s'\n", psz_Varname);
	else    printf ("\t# no name given\n");
	if (psz_Description != 0)
		printf ("\t# Description: '%s'\n", psz_Description);
	else    printf ("\t# no description given\n");
#endif
	if ((p_Cfg == 0) || (p_Cfg->n_Error != CFGERROR_OK))
	{
		return (0);
	}
	if (CfgSetToVariable (p_Cfg, psz_Varname) == 0)
	{
		return (0);
	}
	if (p_Cfg->p_CurrentVariable->psz_Description != 0)
	{
		free (p_Cfg->p_CurrentVariable->psz_Description);
		p_Cfg->p_CurrentVariable->psz_Description = 0;
	}
	i_CfgStrCpy (&(p_Cfg->p_CurrentVariable->psz_Description),
		psz_Description);
	return (1);
} /* CfgSetVarDescriptionOf() */

/**********************************************************************/
/** API-Function
 *
 * find a variable named 'psz_Varname' and set a new comment for it
 *
 * defined in 'cconfig.c'
 *
 */
/**********************************************************************/
int
CfgSetVarCommentOf
	(
	t_config*	p_Cfg,
	char*		psz_Varname,
	char*		psz_Comment
	)
{
#if defined CCFG_DEBUG
	printf ("CfgSetVarCommentOf: start\n");
	if (psz_Varname != 0)
		printf ("\t# Name: '%s'\n", psz_Varname);
	else    printf ("\t# no name given\n");
	if (psz_Comment != 0)
		printf ("\t# Comment: '%s'\n", psz_Comment);
	else    printf ("\t# no comment given\n");
#endif
	if ((p_Cfg == 0) || (p_Cfg->n_Error != CFGERROR_OK))
	{
		return (0);
	}
	if (CfgSetToVariable (p_Cfg, psz_Varname) == 0)
	{
		return (0);
	}
	if (p_Cfg->p_CurrentVariable->psz_Comment != 0)
	{
		free (p_Cfg->p_CurrentVariable->psz_Comment);
		p_Cfg->p_CurrentVariable->psz_Comment = 0;
	}
	i_CfgStrCpy (&(p_Cfg->p_CurrentVariable->psz_Comment),
		psz_Comment);
	return (1);
} /* CfgSetVarCommentOf() */

/**********************************************************************/
/** API-Function
 *
 * find a variable named 'psz_Varname' and set a new status for it
 *
 * defined in 'cconfig.c'
 *
 */
/**********************************************************************/
int
CfgSetVarStatusOf
	(
	t_config*	p_Cfg,
	char*		psz_Varname,
	i_CFGSTATUS	n_Status
	)
{
#if defined CCFG_DEBUG
	printf ("CfgSetVarStatusOf: start\n");
	if (psz_Varname != 0)
		        printf ("\t# Name: '%s'\n", psz_Varname);
	else    printf ("\t# no name given\n");
	printf ("\t# Status: %d\n", n_Status);
#endif
	if ((p_Cfg == 0) || (p_Cfg->n_Error != CFGERROR_OK))
	{
		return (0);
	}
	if (CfgSetToVariable (p_Cfg, psz_Varname) == 0)
	{
		return (0);
	}
	p_Cfg->p_CurrentVariable->n_Status = n_Status;
	return (1);
} /* CfgSetVarStatusOf() */

/**********************************************************************/
/** API-Function
 *
 * find a variable name 'psz_Varname' and set a new name for it
 *
 * defined in 'cconfig.c'
 *
 */
/**********************************************************************/
int
CfgSetVarNameOf
	(
	t_config*	p_Cfg,
	char*		psz_Varname,
	char*		psz_NewName
	)
{
#if defined CCFG_DEBUG
	printf ("CfgSetVarNameOf: start\n");
	if (psz_Varname != 0)
		printf ("\t# Name: '%s'\n", psz_Varname);
	else    printf ("\t# no name given\n");
	if (psz_NewName != 0)
		printf ("\t# NewName: '%s'\n", psz_NewName);
	else    printf ("\t# no new name given\n");

#endif
	if ((p_Cfg == 0) || (p_Cfg->n_Error != CFGERROR_OK))
	{
		return (0);
	}
	if (CfgSetToVariable (p_Cfg, psz_Varname) == 0)
	{
		return (0);
	}
	if (p_Cfg->p_CurrentVariable->psz_Name != 0)
	{
		free (p_Cfg->p_CurrentVariable->psz_Name);
		p_Cfg->p_CurrentVariable->psz_Name = 0;
	}
	i_CfgStrCpy (&(p_Cfg->p_CurrentVariable->psz_Name),
		psz_NewName);
	return (1);
} /* CfgSetVarNameOf() */

/**********************************************************************/
/** API-Function
 *
 * find a variable named 'psz_Varname' and set new predefs for it
 * FIXME: not implemented yet
 *
 * defined in 'cconfig.c'
 *
 */
/**********************************************************************/
int
CfgSetVarPredefOf
	(
	t_config*	p_Cfg,
	char*		psz_Varname,
	char*		psz_Predef
	)
{
#if defined CCFG_DEBUG
	printf ("CfgSetVarPredefOf(): not implemented yet\n");
	if (psz_Varname != 0)
		printf ("\t# Name: '%s'\n", psz_Varname);
	else    printf ("\t# no name given\n");
#endif
	if ((p_Cfg == 0) || (p_Cfg->n_Error != CFGERROR_OK))
	{
		return (0);
	}
	if (CfgSetToVariable (p_Cfg, psz_Varname) == 0)
	{
		return (0);
	}
	if (p_Cfg->p_CurrentVariable->p_Predefs == 0)
		p_Cfg->p_CurrentVariable->p_Predefs = CfgStack_Create ();
	if (psz_Predef != 0)
		CfgStack_Push (p_Cfg->p_CurrentVariable->p_Predefs,
			psz_Predef);
	return (0);
} /* CfgSetVarPredefOf() */

/**********************************************************************/
/** API-Function
 *
 * store all variables in p_Cfg
 *
 * defined in 'cconfig.c'
 *
 * @see CfgRestore
 *
 */
/**********************************************************************/
int
CfgStore
	(
	t_config*	p_Cfg
	)
{
	t_CfgStoreData*		p_StoreData;

#if defined CCFG_DEBUG
	printf ("CfgStore(): start\n");
#endif
	if ((p_Cfg == 0) || (p_Cfg->n_Error != CFGERROR_OK))
	{
		return (1);
	}
	p_StoreData = (t_CfgStoreData*)
		malloc (sizeof (t_CfgStoreData));
	if (p_StoreData == 0)
	{
		return (1);
	}
	if (p_Cfg->p_StoreStack == 0)
		p_Cfg->p_StoreStack = CfgStack_Create ();
	p_StoreData->p_CurrentVariable = p_Cfg->p_CurrentVariable;
	p_StoreData->p_CurrentSection = p_Cfg->p_CurrentSection;
	p_StoreData->p_FirstCurrentVariable = p_Cfg->p_FirstCurrentVariable;
	p_StoreData->p_FirstCurrentSection = p_Cfg->p_FirstCurrentSection;
	p_StoreData->c_Seperator = p_Cfg->c_Seperator;
	CfgStack_Push (p_Cfg->p_StoreStack, p_StoreData);
	return (0);
}

/**********************************************************************/
/** API-Function
 *
 * restore all variables in p_Cfg from values previously
 * stored by CfgStore.
 *
 * defined in 'cconfig.c'
 *
 * @see CfgStore
 *
 */
/**********************************************************************/
int
CfgRestore
	(
	t_config*	p_Cfg
	)
{
	t_CfgStoreData*		p_StoreData;

#if defined CCFG_DEBUG
	printf ("CfgRestore(): start\n");
#endif
	if ((p_Cfg == 0) || (p_Cfg->n_Error != CFGERROR_OK))
	{
		return (1);
	}
	p_StoreData = CfgStack_Pop (p_Cfg->p_StoreStack);
	if (p_StoreData == 0)
	{
		return (1);
	}
	p_Cfg->p_CurrentVariable = p_StoreData->p_CurrentVariable;
	p_Cfg->p_CurrentSection = p_StoreData->p_CurrentSection;
	p_Cfg->p_FirstCurrentVariable = p_StoreData->p_FirstCurrentVariable;
	p_Cfg->p_FirstCurrentSection = p_StoreData->p_FirstCurrentSection;
	p_Cfg->c_Seperator = p_StoreData->c_Seperator;
	free (p_StoreData);
	return (1);
}

/**********************************************************************/
/** API-Function
 *
 * return the name of the next variable in the list
 *
 * defined in 'cconfig.c'
 *
 */
/**********************************************************************/
char*
CfgGetVarNameOfNext
	(
	t_config* p_Cfg
	)
{
#if defined CCFG_DEBUG
	printf ("CfgGetVarNameOfNext(): start\n");
#endif
	if ((p_Cfg == 0) || (p_Cfg->n_Error != CFGERROR_OK))
	{
		return (0);
	}
	if ((p_Cfg->p_CurrentVariable != 0)
	&&  (p_Cfg->p_CurrentVariable->p_Next != 0))
	{
		return (p_Cfg->p_CurrentVariable->p_Next->psz_Name);
	}
	return (0);
} /* CfgGetVarNameOf() */

/**********************************************************************/
/** API-Function
 *
 * return the number of variables of the current context
 *
 * defined in 'cconfig.c'
 *
 */
/**********************************************************************/
int
CfgGetVarCount
	(
	t_config*	p_Cfg
	)
{
	t_variable*	p_Var;
	int		n_Count;
	
#if defined CCFG_DEBUG
	printf ("CfgGetVarCount(): start\n");
#endif
	if ((p_Cfg == 0) || (p_Cfg->n_Error != CFGERROR_OK))
	{
		return (0);
	}
	n_Count = 0;
	p_Var = p_Cfg->p_FirstCurrentVariable;
	while (p_Var != 0)
	{
		n_Count++;
		p_Var = p_Var->p_Next;
	}
	return (n_Count);
} /* CfgGetVarCount() */

/**********************************************************************/
/** API-Function
 *
 * return the name of the current variable
 *
 * defined in 'cconfig.c'
 *
 */
/**********************************************************************/
char*
CfgGetVarName
	(
	t_config*	p_Cfg
	)
{
#if defined CCFG_DEBUG
	printf ("CfgGetVarName(): start\n");
#endif
	if (!i_CfgCurrentVariableExists (p_Cfg))
	{
		return (0);
	}
	return (p_Cfg->p_CurrentVariable->psz_Name);
} /* CfgGetVarName() */

/**********************************************************************/
/** API-Function
 *
 * return the value of the current variable
 *
 * defined in 'cconfig.c'
 *
 */
/**********************************************************************/
char*
CfgGetVarValue
	(
	t_config*	p_Cfg
	)
{
#if defined CCFG_DEBUG
	printf ("CfgGetVarValue(): start\n");
#endif
	if (!i_CfgCurrentVariableExists (p_Cfg))
	{
		return (0);
	}
	if (IsReadable (p_Cfg, p_Cfg->p_CurrentSection->n_Status) == 0)
	{
		return (0);
	}
	return (p_Cfg->p_CurrentVariable->psz_Value);
} /* CfgGetVarValue() */

/**********************************************************************/
/** API-Function
 *
 * return the description of the current variable
 *
 * defined in 'cconfig.c'
 *
 */
/**********************************************************************/
char*
CfgGetVarDescription
	(
	t_config*	p_Cfg
	)
{
#if defined CCFG_DEBUG
	printf ("CfgGetVarDescriction(): start\n");
#endif
	if (!i_CfgCurrentVariableExists (p_Cfg))
	{
		return (0);
	}
	return (p_Cfg->p_CurrentVariable->psz_Description);
} /* CfgGetVarDescription() */

/**********************************************************************/
/** API-Function
 *
 * return the comment of the current variable
 *
 * defined in 'cconfig.c'
 *
 */
/**********************************************************************/
char*
CfgGetVarComment
	(
	t_config*	p_Cfg
	)
{
#if defined CCFG_DEBUG
	printf ("CfgGetVarComment(): start\n");
#endif
	if (!i_CfgCurrentVariableExists (p_Cfg))
	{
		return (0);
	}
	return (p_Cfg->p_CurrentVariable->psz_Comment);
} /* CfgGetVarComment() */

/**********************************************************************/
/** API-Function
 *
 * return the status of the current variable
 *
 * defined in 'cconfig.c'
 *
 */
/**********************************************************************/
int
CfgGetVarStatus
	(
	t_config*	p_Cfg
	)
{
#if defined CCFG_DEBUG
	printf ("CfgGetVarStatus(): start\n");
#endif
	if (!i_CfgCurrentVariableExists (p_Cfg))
	{
		return (CFGERROR_NO_VARIABLE);
	}
	return (p_Cfg->p_CurrentVariable->n_Status);
} /* CfgGetVarStatus() */

/**********************************************************************/
/** API-Function
 *
 * return the predefs of p_Cfg->p_CurrentVariable
 * FIXME: not implemented yet
 *
 * defined in 'cconfig.c'
 *
 */
/**********************************************************************/
char*
CfgGetVarNextPredef
	(
	t_config*	p_Cfg
	)
{
#if defined CCFG_DEBUG
	printf ("CfgGetVarNextPredef(): not implemented yet\n");
#endif
	if (!i_CfgCurrentVariableExists (p_Cfg))
	{
		return (0);
	}
	return (CfgStack_Next (p_Cfg->p_CurrentVariable->p_Predefs));
} /* CfggetVarNextPredef() */

/**********************************************************************/
/** API-Function
 *
 * set the name of p_Cfg->p_CurrentVariable
 *
 * defined in 'cconfig.c'
 *
 */
/**********************************************************************/
int
CfgSetVarName
	(
	t_config*	p_Cfg,
	char*		psz_Name
	)
{
#if defined CCFG_DEBUG
	printf ("CfgSetVarName(): start\n");
	if (psz_Name != 0)
		printf ("\t# Name: '%s'\n", psz_Name);
	else    printf ("\t# no name given\n");
#endif
	if (!i_CfgCurrentVariableExists (p_Cfg))
	{
		return (CFGERROR_NO_VARIABLE);
	}
	if (p_Cfg->p_CurrentVariable->psz_Name != 0)
	{
		free (p_Cfg->p_CurrentVariable->psz_Name);
		p_Cfg->p_CurrentVariable->psz_Name = 0;
	}
	return (i_CfgStrCpy (&(p_Cfg->p_CurrentVariable->psz_Name),
		psz_Name));
} /* CfgSetVarName() */

/**********************************************************************/
/** API-Function
 *
 * set the value of p_Cfg->p_CurrentVariable
 *
 * defined in 'cconfig.c'
 *
 */
/**********************************************************************/
int
CfgSetVarValue 
	(
	t_config*	p_Cfg,
	char*		psz_Value
	)
{
#if defined CCFG_DEBUG
	printf ("CfgSetVarValue(): start\n");
	if (psz_Value != 0)
		printf ("\t# Name: '%s'\n", psz_Value);
	else    printf ("\t# no name given\n");
#endif
	if (!i_CfgCurrentVariableExists (p_Cfg))
	{
		return (CFGERROR_NO_VARIABLE);
	}
	if (p_Cfg->p_CurrentVariable->psz_Value != 0)
	{
		free (p_Cfg->p_CurrentVariable->psz_Value);
		p_Cfg->p_CurrentVariable->psz_Value = 0;
	}
	return (i_CfgStrCpy (&(p_Cfg->p_CurrentVariable->psz_Value),
		psz_Value));
} /* CfgSetVarValue() */

/**********************************************************************/
/** API-Function
 *
 * set the description of p_Cfg->p_CurrentVariable
 *
 * defined in 'cconfig.c'
 *
 */
/**********************************************************************/
int
CfgSetVarDescription
	(
	t_config*	p_Cfg,
	char*		psz_Description
	)
{
#if defined CCFG_DEBUG
	printf ("CfgSetVarDescription(): start\n");
	if (psz_Description != 0)
		printf ("\t# Name: '%s'\n", psz_Description);
	else    printf ("\t# no name given\n");
#endif
	if (!i_CfgCurrentVariableExists (p_Cfg))
	{
		return (CFGERROR_NO_VARIABLE);
	}
	if (p_Cfg->p_CurrentVariable->psz_Description != 0)
	{
		free (p_Cfg->p_CurrentVariable->psz_Description);
		p_Cfg->p_CurrentVariable->psz_Description = 0;
	}
	return(i_CfgStrCpy(&(p_Cfg->p_CurrentVariable->psz_Description),
		psz_Description));
} /* CfgSetVarDescription() */

/**********************************************************************/
/** API-Function
 *
 * set comment of p_Cfg->p_CurrentVariable
 *
 * defined in 'cconfig.c'
 *
 */
/**********************************************************************/
int
CfgSetVarComment
	(
	t_config*	p_Cfg,
	char*		psz_Comment
	)
{
#if defined CCFG_DEBUG
	printf ("CfgSetVarComment(): start\n");
	if (psz_Comment != 0)
		printf ("\t# Name: '%s'\n", psz_Comment);
	else    printf ("\t# no name given\n");
#endif
	if (!i_CfgCurrentVariableExists (p_Cfg))
	{
		return (CFGERROR_NO_VARIABLE);
	}
	if (p_Cfg->p_CurrentVariable->psz_Comment != 0)
	{
		free (p_Cfg->p_CurrentVariable->psz_Comment);
		p_Cfg->p_CurrentVariable->psz_Comment = 0;
	}
	return (i_CfgStrCpy (&(p_Cfg->p_CurrentVariable->psz_Comment),
		psz_Comment));
} /* CfgSetVarComment() */

/**********************************************************************/
/** API-Function
 *
 * set status of p_Cfg->p_CurrentVariable
 *
 * defined in 'cconfig.c'
 *
 */
/**********************************************************************/
int   
CfgSetVarStatus
	(
	t_config*	p_Cfg,
	i_CFGSTATUS	n_Status
	)
{
#if defined CCFG_DEBUG
	printf ("CfgSetVarStatus(%d): start\n", n_Status);
#endif
	if (!i_CfgCurrentVariableExists (p_Cfg))
	{
		return (CFGERROR_NO_VARIABLE);
	}
	p_Cfg->p_CurrentVariable->n_Status = n_Status;
	return (CFGERROR_OK);
} /* CfgSetVarStatus() */

/**********************************************************************/
/** API-Function
 *
 * set predefs of p_Cfg->p_CurrentVariable
 * FIXME: not implemented yet
 *
 * defined in 'cconfig.c'
 *
 */
/**********************************************************************/
int
CfgSetVarPredef
	(
	t_config*	p_Cfg,
	char*		psz_Predef
	)
{
#if defined CCFG_DEBUG
	printf ("CfgSetVarPredef(): enter\n");
	if (psz_Predef != 0)
		printf ("\t# Name: '%s'\n", psz_Predef);
	else    printf ("\t# no name given\n");
#endif
	if (!i_CfgCurrentVariableExists (p_Cfg))
	{
		return (CFGERROR_NO_VARIABLE);
	}
	if (p_Cfg->p_CurrentVariable->p_Predefs == 0)
		p_Cfg->p_CurrentVariable->p_Predefs = CfgStack_Create ();
	if (psz_Predef != 0)
		CfgStack_Push (p_Cfg->p_CurrentVariable->p_Predefs,
			psz_Predef );
	return (0);
} /* CfgSetVarPredef() */
