/*
**	This file is part of cfgd
**
**      Copyright 2002 Oliver Schroeder
**
**      Permission to use, copy, modify, and distribute this
**      software and its documentation for any purpose and
**      without any fee is hereby granted, provided that the
**      above copyright notice and this permission notice
**      appear in all copies of the software and derivative
**      works or modified versions thereof, and that both the
**      copyright notice and this permission and disclaimer
**      notice appear in supporting documentation.
**
**      This software is provided "as is" WITHOUT ANY WARRANTY.
**
**      Oliver Schroeder <post@o-schroeder.de>
**
*/

/**********************************************************************
 * this file contains functions needed by Cfg-API but are not
 * part of the API itself
 **********************************************************************/

/**********************************************************************/
/** internal function
 *
 * copy string psz_Src to ppsz_Dest. allocate memory for 'ppsz_Dest'
 *
 * defined in 'cfghelpers.c'
 *
 * @return	CFGERROR_OUT_OF_MEMORY if no memory is availible
 * @return	CFGERROR_OK if src is NULL
 * @return	CFGERROR_OK if copy was successful
 *
 */
/**********************************************************************/
int
i_CfgStrCpy
	(
	char**	ppsz_Dest,
	char*	psz_Src
	)
{
	int	n_Len;
	char*	psz_NewStr;

	if (psz_Src == 0)
	{
		return (CFGERROR_OK);
	}
	n_Len	= strlen (psz_Src);
	psz_NewStr = (char*) malloc (n_Len + 1);
	if (psz_NewStr == 0)
	{
		return (CFGERROR_OUT_OF_MEMORY);
	}
	memset (psz_NewStr, 0, n_Len + 1);
	strncpy (psz_NewStr, psz_Src, n_Len);
	(*ppsz_Dest) = psz_NewStr;
	return (CFGERROR_OK);
} /* i_CfgStrCpy() */

/**********************************************************************/
/** internal function
 *
 * copy 'size' characters from string 'psz_Src' to 'ppsz_Dest'
 * allocate memory for 'ppsz_Dest'
 *
 * defined in 'cfghelpers.c'
 *
 * @return	CFGERROR_OUT_OF_MEMORY if no memory is availible
 * @return	CFGERROR_OK if src is NULL
 * @return	CFGERROR_OK if copy was successful
 *
 */
/**********************************************************************/
int
i_CfgStrNCpy
	( 
	char**	ppsz_Dest,
	char*	psz_Src,
	size_t	n_Size
	)
{
	int	n_Len;
	char*	psz_NewStr;

	if (psz_Src == 0)
	{
		return (CFGERROR_OK);
	}
	n_Len	= strlen (psz_Src);
	psz_NewStr = (char*) malloc (n_Size + 1);
	if (psz_NewStr == 0)
	{
		return (CFGERROR_OUT_OF_MEMORY);
	}
	if (n_Size > n_Len)
	{
		n_Size = n_Len;
	}
	memset (psz_NewStr, 0, n_Size + 1);
	strncpy (psz_NewStr, psz_Src, n_Size);
	if ((*ppsz_Dest) != 0)
	{
		free ((*ppsz_Dest));
	}
	(*ppsz_Dest) = psz_NewStr;
	return (CFGERROR_OK);
} /* i_CfgStrNCpy() */

/**********************************************************************/
/** internal function
 *
 * concatenate ppsz_Str1 and ppsz_Str2. the resulting string is stored
 * in Str1. this function automatically allocates enough memory for
 * Str1 to store all characters.
 *
 * defined in 'cfghelpers.c'
 *
 * @return	CFGERROR_OUT_OF_MEMORY: no memory is availible
 * @return	CFGERROR_OK: if src is NULL
 * @return	CFGERROR_OK: if copy was successful
 *
 */
/**********************************************************************/
int
i_CfgStrCat 
	(
	char**	ppsz_Str1,
	char*	ppsz_Str2
	)
{
	int	n_LenStr1;
	int	n_LenStr2;
	char*	psz_ResStr;
	char*	psz_Tmp;

	if (ppsz_Str2 == 0)
	{
		return (CFGERROR_OK);
	}
	if ((*ppsz_Str1) != 0)
	{
		n_LenStr1 = strlen((*ppsz_Str1));
	}
	else
	{
		n_LenStr1 = 0;
	}
	n_LenStr2 = strlen(ppsz_Str2);
	psz_ResStr = (char*) malloc (n_LenStr1 + n_LenStr2 + 1);
	if (psz_ResStr == 0)
	{
		return (CFGERROR_OUT_OF_MEMORY);
	}
	memset (psz_ResStr, 0, n_LenStr1 + n_LenStr2 + 1);
	if ((*ppsz_Str1) != 0)
	{
		strncpy (psz_ResStr, (*ppsz_Str1), n_LenStr1);
	}
	psz_Tmp = psz_ResStr + n_LenStr1;
	strncpy (psz_Tmp, ppsz_Str2, n_LenStr2);
	if ((*ppsz_Str1) != 0)
	{
		free ((*ppsz_Str1)); 
	}
	(*ppsz_Str1) = psz_ResStr;
	return (CFGERROR_OK);
}

/********************************************************************/
/** internal function
 *
 * trim string, ie. remove all leading and trailing whitespaces
 *
 * defined in 'cfghelpers.c'
 *
 * @return	pointer to a char-array: OK
 * @return	0: something went wrong
 */
/********************************************************************/
char*
i_CfgTrimString
	(
	char*	psz_Str
	)
{
	char*	psz_Start;
	char*	psz_End;
	char*	psz_NewStr = 0;

	if (psz_Str == 0)
	{
		return (0);
	}
	psz_Start = psz_Str;
	while (isspace(*psz_Start) && (*psz_Start != '\0'))
	{
		psz_Start++;
	}
	psz_End = psz_Str + strlen (psz_Str);
	while ((psz_End > psz_Start) && isspace(*(psz_End-1)))
	{
		psz_End--;
	}
	if (psz_End <= psz_Start)
	{
		return (0);
	}
	i_CfgStrNCpy (&psz_NewStr, psz_Start, psz_End-psz_Start);
	return (psz_NewStr);
}

/**********************************************************************/
/** internal function
 *
 * test if n_BitField (=Cfg->*->Status) allows read actions.
 *
 * defined in 'cfghelpers.c'
 *
 * @return 1: read allowed
 * @return 0: read not allowed
 *
 */
/**********************************************************************/
int
IsReadable
	(
	t_config*	p_Cfg,
	int		n_BitField
	)
{
#ifndef DEVEL
	return (1);	/* is readable */
#else
	if ( ((n_BitField & CFGSTATUS_ACTIVE) == 0)
        ||   ((n_BitField & CFGSTATUS_TEMPORARILY_DEACTIVATED) == 0))
	{
		return (0);
	}
	return (1);
#endif
}

/**********************************************************************/
/** internal function
 *
 * test if n_BitFields (=Cfg->*->Status) allows write actions.
 *
 * defined in 'cfghelpers.c'
 *
 * @return 1: write allowed
 * @return 0: write not allowed
 *
 */
/**********************************************************************/
int
IsWriteable
	(
	t_config*	p_Cfg,
	int		n_BitField 
	)
{
#ifndef DEVEL
	return (1); /* is writeable */
#else
	if ( ((n_BitField & CFGSTATUS_ACTIVE) == 0)
	||   ((n_BitField & CFGSTATUS_ALLOW_USER) != 0)
        ||   ((n_BitField & CFGSTATUS_TEMPORARILY_DEACTIVATED) != 0))
	{
		return (0);
	}
	if ((p_Cfg->n_IsLocal == 1)
	&&  ((n_BitField & CFGSTATUS_ALLOW_OTHER_HOSTS) != 0))
		return (0);
	return (1);
#endif
}

/**********************************************************************/
/** internal function
 *
 * free the 'variables'
 *
 * defined in 'cfghelpers.c'
 *
 */
/**********************************************************************/
void
i_CfgFreeVariables
	(
	t_variable*	p_Variables
	)
{
	t_variable*	p_Current;

	while (p_Variables != 0)
	{
		p_Current = p_Variables->p_Next;
		free (p_Variables->psz_Name);
		free (p_Variables->psz_Value);
		free (p_Variables->psz_Description);
		free (p_Variables->psz_Comment);
		CfgStack_Destroy (p_Variables->p_Predefs);
		free (p_Variables);
		p_Variables = p_Current;
	} /* while */
} /* CfgFreeVriables() */

/**********************************************************************/
/** internal function
 *
 * free 'p_Section'. including variables and p_Subsections.
 *
 * defined in 'cfghelpers.c'
 *
 */
/**********************************************************************/
void
i_CfgFreeSection
	(
	t_section*	p_Section
	)
{
	t_section*	p_Current;

	while (p_Section != 0)
	{
		p_Current = p_Section->p_Next;
		free (p_Section->psz_Name);
		free (p_Section->psz_Description);
		free (p_Section->psz_Comment);
		i_CfgFreeVariables (p_Section->p_Variables);
		i_CfgFreeSection (p_Section->p_Subsections);
		free (p_Section);
		p_Section = p_Current;
	}
	p_Section = 0;
} /* CfgFreeSection() */

/**********************************************************************/
/** internal function
 *
 * create a new section.
 *
 * defined in 'cfghelpers.c'
 *
 */
/**********************************************************************/
t_section*
i_CfgCreateSection
	(
	t_config*	p_Cfg, 
	char*		psz_Name,
	char*		psz_Description,
	char*		psz_Comment,
	i_CFGSTATUS	n_Status
	)
{
	t_section*	p_NewSection;

	if (p_Cfg == 0)
	{
		return (0);
	}
	p_NewSection = (t_section*) malloc (sizeof (t_section));
	if (p_NewSection == 0)
	{
		return (0);
	}
	memset (p_NewSection, 0, sizeof (t_section));
	i_CfgStrCpy (&(p_NewSection->psz_Name), psz_Name);
	i_CfgStrCpy (&(p_NewSection->psz_Description), psz_Description);
	i_CfgStrCpy (&(p_NewSection->psz_Comment), psz_Comment);
	p_NewSection->n_Status	= n_Status;
	p_NewSection->p_Variables	= 0;
	p_NewSection->p_Next	= 0;
	p_NewSection->p_Prev	= 0;
	return (p_NewSection);
} /* i_CfgCreateSection() */

/**********************************************************************/
/** internal function
 *
 * wind to the last section in section list
 *
 * defined in 'cfghelpers.c'
 *
 */
/**********************************************************************/
void
i_CfgEndOfSections
	(
	t_config*	p_Cfg
	)
{
	t_section*	p_Sec;
	
	if ((p_Cfg == 0) || (p_Cfg->p_CurrentSection == 0))
	{
		return;
	}
	p_Sec = p_Cfg->p_CurrentSection;
	while (p_Sec->p_Next != 0)
	{
		p_Sec = p_Sec->p_Next;
	}
	p_Cfg->p_CurrentSection		= p_Sec;
	p_Cfg->p_CurrentVariable	= p_Sec->p_Variables;
	p_Cfg->p_FirstCurrentVariable	= p_Sec->p_Variables;
} /* i_CfgEndOfSections() */

/**********************************************************************/
/** internal function
 *
 * wind to the last variable in variable list
 *
 * defined in 'cfghelpers.c'
 *
 */
/**********************************************************************/
void
i_CfgEndOfVariables
	(
	t_config*	p_Cfg
	)
{
	t_variable*	p_Var;
	
	if ((p_Cfg == 0) || (p_Cfg->p_CurrentVariable == 0))
	{
		return;
	}
	p_Var = p_Cfg->p_CurrentVariable;
	while (p_Var->p_Next != 0)
	{
		p_Var = p_Var->p_Next;
	}
	p_Cfg->p_CurrentVariable = p_Var;
} /* i_CfgEndOfVariables() */

/**********************************************************************/
/** internal function
 *
 * test if p_CurrentSection is not 0
 *
 * defined in 'cfghelpers.c'
 *
 */
/**********************************************************************/
int
i_CfgCurrentSectionExists
	(
	t_config*	p_Cfg
	)
{
	if (p_Cfg == 0)
	{
		return (0);
	}
	if (p_Cfg->p_CurrentSection == 0)
	{
		p_Cfg->n_Error = CFGERROR_NO_SECTION;
		return (0);
	}
	return (1);
} /* i_CfgCurrentSectionExists() */

/**********************************************************************/
/** internal function
 *
 * test if p_CurrentVariable is not 0
 *
 */
/**********************************************************************/
int
i_CfgCurrentVariableExists
	(
	t_config*	p_Cfg
	)
{
	if (p_Cfg == 0)
	{
		return (0);
	}
	if (p_Cfg->p_CurrentVariable == 0)
	{
		p_Cfg->n_Error = CFGERROR_NO_SECTION;
		return (0);
	}
	return (1);
} /* i_CfgCurrentVariableExists() */

/**********************************************************************/
/** internal function
 *
 * write 'Count' * tab to 'Target'
 *
 * defined in 'cfghelpers.c'
 *
 */
/**********************************************************************/
void
i_CfgWriteTabs
	( 
	FILE*	p_Target,
	int	n_Count
	)
{
	int n_I;

	for (n_I = 0; n_I < n_Count; n_I++)
	{
//		fprintf (p_Target, "\t");
		printf ("\t");
	}
} /* i_CfgWriteTabs() */

/**********************************************************************/
/** internal function
 *
 * write all variables to 'Target'
 *
 * defined in 'cfghelpers.c'
 *
 */
/**********************************************************************/
void
i_CfgWriteVariablesEx
	( 
	FILE*		p_Target,
	t_variable*	p_Variable,
	int 		n_Depth
	)
{
	char*	psz_TmpStr;

	i_CfgWriteTabs (p_Target, n_Depth);
	printf ("<var>\n");
	if (p_Variable->psz_Name != 0)
	{
		i_CfgWriteTabs (p_Target, n_Depth+1);
		printf ("<name>%s</name>\n",
			p_Variable->psz_Name);
	}
	if (p_Variable->psz_Value != 0)
	{
		i_CfgWriteTabs (p_Target, n_Depth+1);
		printf ("<value>%s</value>\n",
			p_Variable->psz_Value);
	}
	if (p_Variable->psz_Description != 0)
	{
		i_CfgWriteTabs (p_Target, n_Depth+1);
		printf ("<description>%s</description>\n",
			p_Variable->psz_Description);
	}
	if (p_Variable->psz_Comment != 0)
	{
		i_CfgWriteTabs (p_Target, n_Depth+1);
		printf ("<comment>%s</comment>\n",
			p_Variable->psz_Comment);
	}
	i_CfgWriteTabs (p_Target, n_Depth+1);
	printf ("<status>%u</status>\n",
		p_Variable->n_Status);
	if (! CfgStack_Empty (p_Variable->p_Predefs))
	{
		i_CfgWriteTabs (p_Target, n_Depth+1);
		printf ("<predef>\n");
		CfgStack_SetStart (p_Variable->p_Predefs);
		psz_TmpStr = CfgStack_Next (p_Variable->p_Predefs);
		n_Depth++;
		while (psz_TmpStr != 0)
		{
			i_CfgWriteTabs (p_Target, n_Depth+1);
			printf ("<value>%s</value>\n",
				psz_TmpStr);
			psz_TmpStr=CfgStack_Next(p_Variable->p_Predefs);
		}
		n_Depth--;
		i_CfgWriteTabs (p_Target, n_Depth+1);
		printf ("</predef>\n");
	}
#if defined CCFG_DEBUG
	i_CfgWriteTabs (p_Target, n_Depth+1);
	if (p_Variable->p_Prev != 0)
	{
		printf ("<prev>%s</prev>\n",
			p_Variable->p_Prev->psz_Name);
	}
	else
	{
		printf ("<prev>null</prev>\n");
	}
	i_CfgWriteTabs (p_Target, n_Depth+1);
	if (p_Variable->p_Next != 0)
	{
		printf ("<next>%s</next>\n",
			p_Variable->p_Next->psz_Name);
	}
	else
	{
		printf ("<next>null</next>\n");
	}
#endif
	i_CfgWriteTabs (p_Target, n_Depth);
	printf ("</var>\n");
} /* i_CfgWriteVariablesEx() */

/**********************************************************************/
/** internal function
 *
 * write all variables to 'Target'
 *
 * defined in 'cfghelpers.c'
 *
 */
/**********************************************************************/
void
i_CfgWriteVariables
	( 
	FILE*		p_Target,
	t_variable*	p_Variable,
	int 		n_Depth
	)
{
	char*	psz_TmpStr;

	while (p_Variable != 0)
	{
		i_CfgWriteTabs (p_Target, n_Depth);
		printf ("<var>\n");
		if (p_Variable->psz_Name != 0)
		{
			i_CfgWriteTabs (p_Target, n_Depth+1);
			printf ("<name>%s</name>\n",
				p_Variable->psz_Name);
		}
		if (p_Variable->psz_Value != 0)
		{
			i_CfgWriteTabs (p_Target, n_Depth+1);
			printf ("<value>%s</value>\n",
				p_Variable->psz_Value);
		}
		if (p_Variable->psz_Description != 0)
		{
			i_CfgWriteTabs (p_Target, n_Depth+1);
			printf ("<description>%s</description>\n",
				p_Variable->psz_Description);
		}
		if (p_Variable->psz_Comment != 0)
		{
			i_CfgWriteTabs (p_Target, n_Depth+1);
			printf ("<comment>%s</comment>\n",
				p_Variable->psz_Comment);
		}
		i_CfgWriteTabs (p_Target, n_Depth+1);
		printf ("<status> %u </status>\n",
			p_Variable->n_Status);
		if (! CfgStack_Empty (p_Variable->p_Predefs))
		{
			i_CfgWriteTabs (p_Target, n_Depth+1);
			printf ("<predef>\n");
			CfgStack_SetStart (p_Variable->p_Predefs);
			psz_TmpStr = CfgStack_Next (p_Variable->p_Predefs);
			n_Depth++;
			while (psz_TmpStr != 0)
			{
				i_CfgWriteTabs (p_Target, n_Depth+1);
				printf ("<value>%s</value>\n",
					psz_TmpStr);
				psz_TmpStr=CfgStack_Next(p_Variable->p_Predefs);
			}
			n_Depth--;
			i_CfgWriteTabs (p_Target, n_Depth+1);
			printf ("</predef>\n");
		}
#if defined CCFG_DEBUG
		i_CfgWriteTabs (p_Target, n_Depth+1);
		if (p_Variable->p_Prev != 0)
		{
			printf ("<prev>%s</prev>\n",
				p_Variable->p_Prev->psz_Name);
		}
		else
		{
			printf ("<prev> null </prev>\n");
		}
		i_CfgWriteTabs (p_Target, n_Depth+1);
		if (p_Variable->p_Next != 0)
		{
			printf ("<next>%s</next>\n",
				p_Variable->p_Next->psz_Name);
		}
		else
		{
			printf ("<next> null </next>\n");
		}
#endif
		i_CfgWriteTabs (p_Target, n_Depth);
		printf ("</var>\n");
		p_Variable = p_Variable->p_Next;
	} /* while */
} /* i_CfgWriteVariables() */

/**********************************************************************/
/** internal function
 *
 * write a section, excluding subsections, not recursivly
 *
 * defined in 'cfghelpers.c'
 *
 */
/**********************************************************************/
void
i_CfgWriteSectionEx
	(
	FILE*		p_Target,
	t_section*	p_Section,
	int		n_Depth
	)
{
	i_CfgWriteTabs (p_Target, n_Depth);
	printf ("<section>\n");
	if (p_Section->psz_Name != 0)
	{
		i_CfgWriteTabs (p_Target, n_Depth+1);
		printf ("<name>%s</name>\n",
			p_Section->psz_Name);
	}
	if (p_Section->psz_Description != 0)
	{
		i_CfgWriteTabs (p_Target, n_Depth+1);
		printf ("<description>%s</description>\n",
			p_Section->psz_Description);
	}
	if (p_Section->psz_Comment != 0)
	{
		i_CfgWriteTabs (p_Target, n_Depth+1);
		printf ("<comment>%s</comment>\n",
			p_Section->psz_Comment);
	}
	if (p_Section->n_Status != 1)
	{
		i_CfgWriteTabs (p_Target, n_Depth+1);
		printf ("<status> %u </status>\n",
			p_Section->n_Status);
	}
#if defined CCFG_DEBUG
	i_CfgWriteTabs (p_Target, n_Depth+1);
	if (p_Section->p_Prev != 0)
	{
		printf ("<prev>%s</prev>\n",
			p_Section->p_Prev->psz_Name);
	}
	else
	{
		printf ("<prev> null </prev>\n");
	}
	i_CfgWriteTabs (p_Target, n_Depth+1);
	if (p_Section->p_Next != 0)
	{
		printf ("<next>%s</next>\n",
			p_Section->p_Next->psz_Name);
	}
	else
	{
		printf ("<next> null </next>\n");
	}
	i_CfgWriteTabs (p_Target, n_Depth+1);
	if (p_Section->p_Parent != 0)
	{
		printf ("<parent>%s</parent>\n",
			p_Section->p_Parent->psz_Name);
	}
	else
	{
		printf ("<parent> null </parent>\n");
	}
#endif
	i_CfgWriteTabs (p_Target, n_Depth);
	printf ("</section>\n");
}

/**********************************************************************/
/** internal function
 *
 * write a section, including subsections, recursivly
 *
 * defined in 'cfghelpers.c'
 *
 */
/**********************************************************************/
void
i_CfgWriteSection
	(
	FILE*		p_Target,
	t_section*	p_Section,
	int		n_Depth
	)
{
	while (p_Section != 0)
	{
		i_CfgWriteTabs (p_Target, n_Depth);
		printf ("<section>\n");
		if (p_Section->psz_Name != 0)
		{
			i_CfgWriteTabs (p_Target, n_Depth+1);
			printf ("<name>%s</name>\n",
				p_Section->psz_Name);
		}
		if (p_Section->psz_Description != 0)
		{
			i_CfgWriteTabs (p_Target, n_Depth+1);
			printf ("<description>%s</description>\n",
				p_Section->psz_Description);
		}
		if (p_Section->psz_Comment != 0)
		{
			i_CfgWriteTabs (p_Target, n_Depth+1);
			printf ("<comment>%s</comment>\n",
				p_Section->psz_Comment);
		}
		i_CfgWriteTabs (p_Target, n_Depth+1);
		printf ("<status> %u </status>\n",
			p_Section->n_Status);
#if defined CCFG_DEBUG
		i_CfgWriteTabs (p_Target, n_Depth+1);
		if (p_Section->p_Prev != 0)
		{
			printf ("<prev>%s</prev>\n",
				p_Section->p_Prev->psz_Name);
		}
		else
		{
			printf ("<prev> null </prev>\n");
		}
		i_CfgWriteTabs (p_Target, n_Depth+1);
		if (p_Section->p_Next != 0)
		{
			printf ("<next>%s</next>\n",
				p_Section->p_Next->psz_Name);
		}
		else
		{
			printf ("<next> null </next>\n");
		}
		i_CfgWriteTabs (p_Target, n_Depth+1);
		if (p_Section->p_Parent != 0)
		{
			printf ("<parent>%s</parent>\n",
				p_Section->p_Parent->psz_Name);
		}
		else
		{
			printf ("<parent> null </parent>\n");
		}
#endif
		i_CfgWriteVariables (p_Target, p_Section->p_Variables,
			n_Depth+1);
		i_CfgWriteSection (p_Target, p_Section->p_Subsections,
			n_Depth+1);
		i_CfgWriteTabs (p_Target, n_Depth);
		printf ("</section>\n");
		p_Section = p_Section->p_Next;
	} /* while */
} /* CfgWriteSection() */

/**********************************************************************/
/** internal function
 * 
 * internal function.
 * recursivly find section by name
 *
 * FIXME: not optimized at all but working
 *
 * defined in 'cfghelpers.c'
 *
 */
/**********************************************************************/
int
i_CfgFindSection
	(
	t_config*	p_Cfg,
	char*		psz_Name
	)
{
	int		n_Len;
	int		n_Ret;
	t_section*	p_CurrentSec;
	t_section*	p_FirstSec;
	t_variable*	p_CurrentVar;
	t_variable*	p_FirstVar;
	t_section*	p_Tmp_CurrentSec;
	t_section*	p_Tmp_FirstSec;

#if defined CCFG_DEBUG
	printf ("i_CfgFindSection start\n");
	if (psz_Name != 0)
		printf ("\t# Name: '%s'", psz_Name);
	else    printf ("\t# no name given\n");
#endif
	if (p_Cfg == 0)
	{
		return (0);
	}
	p_CurrentSec	= p_Cfg->p_CurrentSection;
	p_FirstSec	= p_Cfg->p_FirstCurrentSection;
	p_CurrentVar	= p_Cfg->p_CurrentVariable;
	p_FirstVar	= p_Cfg->p_FirstCurrentVariable;
	n_Len		= strlen (psz_Name);
	n_Ret		= 0;
	while (p_Cfg->p_CurrentSection != 0)
	{
		if (strncmp(p_Cfg->p_CurrentSection->psz_Name,
			psz_Name,n_Len)==0)
		{
			p_Cfg->p_CurrentVariable = 
				p_Cfg->p_CurrentSection->p_Variables;
			return (1);
		}
		if (p_Cfg->p_CurrentSection->p_Subsections != 0)
		{
			p_Tmp_CurrentSec = p_Cfg->p_CurrentSection;
			p_Tmp_FirstSec	 = p_Cfg->p_FirstCurrentSection;
			CfgNextSubSection (p_Cfg);
			n_Ret = i_CfgFindSection (p_Cfg, psz_Name);
			if (n_Ret != 0)
			{	/* found */
				p_Cfg->p_FirstCurrentVariable =
				  p_Cfg->p_CurrentSection->p_Variables;
				p_Cfg->p_CurrentVariable =
				  p_Cfg->p_CurrentSection->p_Variables;
				return (n_Ret);
			}
			p_Cfg->p_FirstCurrentSection = p_Tmp_FirstSec;
			p_Cfg->p_CurrentSection	= p_Tmp_CurrentSec;
		} /* if */
		CfgNextSection (p_Cfg);
	} /* while */
	p_Cfg->p_CurrentSection		= p_CurrentSec;
	p_Cfg->p_FirstCurrentSection	= p_FirstSec;
	p_Cfg->p_CurrentVariable	= p_CurrentVar;
	p_Cfg->p_FirstCurrentVariable	= p_FirstVar;
	return (0);
} /* i_CfgFindSection() */


/**********************************************************************/
/** internal function
 *
 * find variable 'psz_Name' in 'p_Cfg'
 *
 * defined in 'cfghelpers.c'
 *
 * @param p_Cfg
 * 	a pointer to a valid t_config
 * @param pszVarname
 * 	of the variable including section-names
 * @precondition
 *	p_Cfg must be valid or this function will simply do nothing
 * @precondition
 *      p_Cfg->n_Error must be CFGERROR_OK or this function will
 *	simply do nothing
 * @return
 * 	0: not found
 * @return
 * 	1: found
 *
 */
/**********************************************************************/
int
i_CfgSetToVariable
	(
	t_config*	p_Cfg,
	char*		psz_Varname
	)
{
	char*   	psz_Start;
	char*   	psz_Pos;
	int     	n_Found;
	int     	n_Len;
	int		n_Ret;
	t_variable*	p_CurrentVar;
	t_variable*	p_FirstVar;
	t_section*	p_CurrentSec;
	t_section*	p_FirstSec;

#if defined CCFG_DEBUG
	printf ("i_CfgSetToVariable start\n");
	if (psz_Varname != 0)
		printf ("\t# Name: '%s\n", psz_Varname);
	else	printf ("\t# no name given\n");
#endif
	if ((p_Cfg == 0) || (p_Cfg->n_Error != CFGERROR_OK))
	{
		return (0);
	}
	p_CurrentVar	= p_Cfg->p_CurrentVariable;
	p_FirstVar	= p_Cfg->p_FirstCurrentVariable;
	p_CurrentSec	= p_Cfg->p_CurrentSection;
	p_FirstSec	= p_Cfg->p_FirstCurrentSection;
	psz_Start	= psz_Varname;
	psz_Pos		= (char*)index(psz_Varname, p_Cfg->c_Seperator);
	if (psz_Pos == 0)
	{
		return (CfgFindVariable (p_Cfg, psz_Varname));
	}
	n_Len   = psz_Pos - psz_Start;
	n_Found = 0;
	while ((n_Found == 0) && (p_Cfg->p_CurrentSection != 0))
	{
		if (strncmp (psz_Start, 
			p_Cfg->p_CurrentSection->psz_Name,n_Len) == 0)
		{
			n_Found = 1;
		}
		else
		{
			CfgNextSection (p_Cfg);
		}
	}
	if (n_Found == 0)
	{
		p_Cfg->p_CurrentVariable	= p_CurrentVar;
		p_Cfg->p_FirstCurrentVariable	= p_FirstVar;
		p_Cfg->p_CurrentSection		= p_CurrentSec;
		p_Cfg->p_FirstCurrentSection	= p_FirstSec;
		return (0);
	}
	/*
	 * one more section to seek?
	 */
	if ((char*) index (psz_Pos+1, p_Cfg->c_Seperator) != 0)
	{
		CfgNextSubSection (p_Cfg);
	}
	n_Ret = i_CfgSetToVariable (p_Cfg, psz_Pos + 1);
	if (n_Ret == 0)
	{
		p_Cfg->p_CurrentVariable	= p_CurrentVar;
		p_Cfg->p_FirstCurrentVariable	= p_FirstVar;
		p_Cfg->p_CurrentSection		= p_CurrentSec;
		p_Cfg->p_FirstCurrentSection	= p_FirstSec;
	}
	return (n_Ret);
} /* i_CfgSetToVariable */

/**********************************************************************/
/** internal function
 *
 * find section 'psz_Name' in 'p_Cfg'
 *
 * defined in 'cfghelpers.c'
 *
 * @param p_Cfg
 * 	a pointer to a valid t_config
 * @param pszSecname
 * 	of the section including parent-p_Sections
 * @precondition
 *	p_Cfg must be valid or this function will simply do nothing
 * @precondition
 *      p_Cfg->n_Error must be CFGERROR_OK or this function will
 *	simply do nothing
 * @return
 * 	0: not found
 * @return
 * 	1: found
 *
 */
/**********************************************************************/
int
i_CfgSetToSection
	(
	t_config*	p_Cfg,
	char*		psz_Secname
	)
{
	char*   	psz_Start;
	char*   	psz_Pos;
	int     	n_Found;
	int     	n_Len;
	int		n_Ret;
	t_variable*	p_CurrentVar;
	t_variable*	p_FirstVar;
	t_section*	p_CurrentSec;
	t_section*	p_FirstSec;

#if defined CCFG_DEBUG
	printf ("i_CfgSetToSection start\n");
	if (psz_Secname != 0)
		printf ("\t# Name: '%s\n", psz_Secname);
	else    printf ("\t# no name given\n");

#endif
	if ((p_Cfg == 0) || (p_Cfg->n_Error != CFGERROR_OK))
	{
		return (0);
	}
	p_CurrentVar	= p_Cfg->p_CurrentVariable;
	p_FirstVar	= p_Cfg->p_FirstCurrentVariable;
	p_CurrentSec	= p_Cfg->p_CurrentSection;
	p_FirstSec	= p_Cfg->p_FirstCurrentSection;
	psz_Start	= psz_Secname;
	psz_Pos		= (char*)index(psz_Secname, p_Cfg->c_Seperator);
	if (psz_Pos == 0)
	{
		return (CfgFindSection (p_Cfg, psz_Secname));
	}
	n_Len   = psz_Pos - psz_Start;
	n_Found = 0;
	while ((n_Found == 0) && (p_Cfg->p_CurrentSection != 0))
	{
		if (strncmp (psz_Start,
			p_Cfg->p_CurrentSection->psz_Name,n_Len) == 0)
		{
			n_Found = 1;
		}
		else
		{
			CfgNextSection (p_Cfg);
		}
	}
	if (n_Found == 0)
	{
		p_Cfg->p_CurrentVariable	= p_CurrentVar;
		p_Cfg->p_FirstCurrentVariable	= p_FirstVar;
		p_Cfg->p_CurrentSection		= p_CurrentSec;
		p_Cfg->p_FirstCurrentSection	= p_FirstSec;
		return (0);
	}
	CfgNextSubSection (p_Cfg);
	n_Ret = i_CfgSetToSection (p_Cfg,psz_Pos+1);
	if (n_Ret == 0)
	{
		p_Cfg->p_CurrentVariable	= p_CurrentVar;
		p_Cfg->p_FirstCurrentVariable	= p_FirstVar;
		p_Cfg->p_CurrentSection		= p_CurrentSec;
		p_Cfg->p_FirstCurrentSection	= p_FirstSec;
	}
	return (n_Ret);
} /* i_CfgSetToVariable */

