/*
**	This file is part of cfgd
**
**      Copyright 2002 Oliver Schroeder
**
**      Permission to use, copy, modify, and distribute this
**      software and its documentation for any purpose and
**      without any fee is hereby granted, provided that the
**      above copyright notice and this permission notice
**      appear in all copies of the software and derivative
**      works or modified versions thereof, and that both the
**      copyright notice and this permission and disclaimer
**      notice appear in supporting documentation.
**
**      This software is provided "as is" WITHOUT ANY WARRANTY.
**
**      Oliver Schroeder <post@o-schroeder.de>
**
*/

/*
 * Elements of a config file:
 * <config version="1.0">			- start of a config
 * <var>					- a variable
 * 	<name> Name </name>			- name of variable
 * 	<value> Value </value>			- content of variable
 * 	<description> Desc </description	- a description for it
 * 	<comment> Comment </comment>		- a (user) comment
 * 	<status> Status </status>		- variable status
 * 	<predef>				- possible values
 * 		<value> val1 </value>		  
 * 		<value> val2 </value>		 
 * 	</predef>
 * </var>
 * <section>					- a section
 *	<name> Name </name>
 * 	<description> Desc </description
 * 	<comment> Comment </comment>
 * 	<status> Status </status>
 *	<var>
 *		...
 *	</var>
 *	<section>
 *		...
 *	</section>
 * </section>
 * </config>					- end of config
 *
 */

typedef struct st_ParserData
{
	t_CfgStack*	p_ParserStack;
	t_config*	p_Config;
	char*		psz_CurrentString;
	int		n_LastAction;
} t_ParserData;

enum i_CfgKnownTags
{
	i_eCFG_KF_UNKNOWN,	/* unknown tag	*/
	i_eCFG_KF_SECTION,	/* <section>	*/
	i_eCFG_KF_VAR,		/* <var>	*/
	i_eCFG_KF_NAME,		/* <name>	*/
	i_eCFG_KF_VALUE,	/* <value>	*/
	i_eCFG_KF_DESCRIPTION,	/* <description>*/
	i_eCFG_KF_COMMENT,	/* <comment>	*/
	i_eCFG_KF_STATUS,	/* <status>	*/
	i_eCFG_KF_PREDEF,	/* <predef>	*/
	i_eCFG_KF_PREDEF_VAL,	/* <predef>	*/
	i_eCFG_KF_CONFIG	/* <config>	*/
};

enum i_CfgLastAction
{
	i_eCFG_LA_NOTHING,
	i_eCFG_LA_STARTTAG,
	i_eCFG_LA_ENDTAG,
	i_eCFG_LA_DATA
};

/********************************************************************/
/** internal function.
 *
 *  sets internal values of current section.
 *
 *  defined in 'cfgparser.c'
 *
 *  @param Context	integer indicating the field of a t_section
 *  			to be manipulated.
 *
 *  @return		nothing
 *
 */
/********************************************************************/
static void
i_CfgParserHandleSection
	(
	t_ParserData*	p_ParserData,
	int		n_Context
	)
{
	int	n_I;
	
#if defined CCFG_PARSER_DEBUG
	printf ("i_CfgParserHandleSection() start\n");
	if (p_ParserData->psz_CurrentString != 0)
		printf ("\t# current string: '%s'\n",
		  p_ParserData->psz_CurrentString);
	else	printf ("\t# current string empty!\n");
#endif
	switch (n_Context)
	{
		case i_eCFG_KF_NAME:
			if ((p_ParserData == 0)
			||  (p_ParserData->psz_CurrentString == 0))
				break;
			CfgSetSectionName (p_ParserData->p_Config,
				p_ParserData->psz_CurrentString);
			break;
		case i_eCFG_KF_DESCRIPTION:
			CfgSetSectionDescription (
				p_ParserData->p_Config,
				p_ParserData->psz_CurrentString);
			break;
		case i_eCFG_KF_COMMENT:
			CfgSetSectionComment (p_ParserData->p_Config,
				p_ParserData->psz_CurrentString);
			break;
		case i_eCFG_KF_STATUS:
			n_I = atoi (p_ParserData->psz_CurrentString);
			CfgSetSectionStatus (p_ParserData->p_Config,
				n_I);
			break;
		default:/*
			 * this is the right place to handle errors.
			 * to support future extensions, we ignore
			 * errors.
			 */
			break;
	} /* switch */
} /* i_CfgParserHandleSection() */

/********************************************************************/
/** internal function
 *
 *  sets internal values of current variable.
 *
 *  defined in 'cfgparser.c'
 *
 *  @param Context	integer indicating the field of a t_variable
 *  			to be manipulated.
 *
 *  @return		nothing
 *
 */
/********************************************************************/
static void
i_CfgParserHandleVariable
	(
	t_ParserData*	p_ParserData,
	int		n_Context
	)
{
	int	n_I;

#if defined CCFG_PARSER_DEBUG
	printf ("i_CfgParserHandleVariable() start\n");
	if (p_ParserData->psz_CurrentString != 0)
		printf ("\t# current string: '%s'\n",
		  p_ParserData->psz_CurrentString);
	else	printf ("\t# current string empty!\n");
#endif
	switch (n_Context)
	{
		case i_eCFG_KF_NAME:
			if ((p_ParserData == 0)
			||  (p_ParserData->psz_CurrentString == 0))
				break;
			CfgSetVarName (p_ParserData->p_Config,
				p_ParserData->psz_CurrentString);
			break;
		case i_eCFG_KF_VALUE:
			CfgSetVarValue (p_ParserData->p_Config,
				p_ParserData->psz_CurrentString);
			break;
		case i_eCFG_KF_DESCRIPTION:
			CfgSetVarDescription (p_ParserData->p_Config,
				p_ParserData->psz_CurrentString);
			break;
		case i_eCFG_KF_COMMENT:
			CfgSetVarComment (p_ParserData->p_Config,
				p_ParserData->psz_CurrentString);
			break;
		case i_eCFG_KF_STATUS:
			n_I = atoi (p_ParserData->psz_CurrentString);
			CfgSetVarStatus (p_ParserData->p_Config, n_I);
			break;
		case i_eCFG_KF_PREDEF_VAL:
			CfgSetVarPredef (p_ParserData->p_Config,
				p_ParserData->psz_CurrentString);
			break;
		default:/*
			 * this is the right place to handle errors.
			 * to support future extensions, we ignore
			 * errors.
			 */
			break;
	} /* switch */
} /* i_CfgParserHandleVariable() */

/********************************************************************/
/** internal function
 *
 *  this is the XML start-tag handler.
 *  called by the parser of the expat library. the parameters are
 *  given by the expat library. because we can not change them, but
 *  have to manipulte a 't_config' object, we have a global variable
 *  called 'p_Config'. to eliminate all global variables we have to 
 *  integrate expat to Cfg and manipulate all expat-functions
 *  accordingly.
 *
 *  defined in 'cfgparser.c'
 *
 */
/********************************************************************/
static void
i_CfgXMLStartTagHandler
	(
	void*		UserData,
	const char*	Name,
	const char**	Atts
	)
{
	int*		pn_CurrentContext;
	int*		pn_PreContext;
	t_ParserData*	p_ParserData;

#if defined CCFG_PARSER_DEBUG
	printf ("i_CfgXMLStartTagHandler () start\n");
	if (Name != 0)
		printf ("\t# name: '%s'\n", Name);
	else	printf ("\t# name empty!\n");
	if (p_ParserData->psz_CurrentString != 0)
		printf ("\t# current string: '%s'\n",
		  p_ParserData->psz_CurrentString);
	else	printf ("\t# current string empty!\n");
#endif
	p_ParserData = (t_ParserData*) UserData;
	pn_CurrentContext = (int*) malloc (sizeof(int));
	pn_PreContext = CfgStack_Top (p_ParserData->p_ParserStack);
	if (strcmp (Name, "section") == 0)
	{
		*pn_CurrentContext = i_eCFG_KF_SECTION;
		if ((pn_PreContext != i_eCFG_KF_UNKNOWN)
		&& (*pn_PreContext != i_eCFG_KF_SECTION))
		{
			CfgAddSection (p_ParserData->p_Config,
				0, 0, 0, 1);
		}
		else
		{
			CfgAddSubSection (p_ParserData->p_Config,
				0, 0, 0, 1);
		}
	}
	else if (strcmp (Name, "var") == 0)
	{
		*pn_CurrentContext = i_eCFG_KF_VAR;
		CfgAddVariable (p_ParserData->p_Config, 0,0,0,0,1);
	}
	else if (strcmp (Name, "name") == 0)
	{
		*pn_CurrentContext = i_eCFG_KF_NAME;
	}
	else if (strcmp (Name, "value") == 0)
	{
		if (*pn_PreContext == i_eCFG_KF_VAR)
			*pn_CurrentContext = i_eCFG_KF_VALUE;
		else if (*pn_PreContext == i_eCFG_KF_PREDEF)
			*pn_CurrentContext = i_eCFG_KF_PREDEF_VAL;
		else	*pn_CurrentContext = i_eCFG_KF_UNKNOWN;

	}
	else if (strcmp (Name, "description") == 0)
	{
		*pn_CurrentContext = i_eCFG_KF_DESCRIPTION;
	}
	else if (strcmp (Name, "comment") == 0)
	{
		*pn_CurrentContext = i_eCFG_KF_COMMENT;
	}
	else if (strcmp (Name, "status") == 0)
	{
		*pn_CurrentContext = i_eCFG_KF_STATUS;
	}
	else if (strcmp (Name, "predef") == 0)
	{
		*pn_CurrentContext = i_eCFG_KF_PREDEF;
		CfgSetVarPredef (p_ParserData->p_Config, 0);
	}
	else if (strcmp (Name, "config") == 0)
	{
		*pn_CurrentContext = i_eCFG_KF_CONFIG;
	}
	else
	{	/* unknown */
		*pn_CurrentContext = i_eCFG_KF_UNKNOWN;
	}
	CfgStack_Push (p_ParserData->p_ParserStack, pn_CurrentContext);
	p_ParserData->n_LastAction = i_eCFG_LA_STARTTAG;
} /* i_CfgXMLStartTagHandler() */

/********************************************************************/
/** internal function
 *
 *  this is the XML end-tag handler.
 *  called by the parser of the expat library. the parameters are
 *  given by the expat library. because we can not change them, but
 *  have to manipulte a 't_config' object, we have a global variable
 *  called 'p_Config'. to eliminate all global variables we have to 
 *  integrate expat to Cfg and manipulate all expat-functions
 *  accordingly.
 *
 *  defined in 'cfgparser.c'
 *
 */
/********************************************************************/
static void
i_CfgXMLEndTagHandler
	(
	void*		UserData,
	const char*	Name
	)
{
	/*
	char*		psz_Str = 0;
	*/
	int*		pn_CurrentContext;
	int*		pn_PreContext;
	t_ParserData*	p_ParserData;

#if defined CCFG_PARSER_DEBUG
	printf ("i_CfgXMLEndTagHandler () start\n");
	if (Name != 0)
		printf ("\t# name: '%s'\n", Name);
	else	printf ("\t# name empty!\n");
	if (p_ParserData->psz_CurrentString != 0)
		printf ("\t# current string: '%s'\n",
		  p_ParserData->psz_CurrentString);
	else	printf ("\t# current string empty!\n");
#endif
	p_ParserData = (t_ParserData*) UserData;
#if 0
	 psz_Str = i_CfgTrimString ((char*) Name);
#endif
	if (!CfgStack_Empty (p_ParserData->p_ParserStack))
	{
		pn_CurrentContext = ((int*) CfgStack_Pop (
			p_ParserData->p_ParserStack));
	} /* if */
	pn_PreContext = ((int*) CfgStack_Top (
		p_ParserData->p_ParserStack));
	if (pn_PreContext != 0)
	{
		switch (*pn_PreContext)
		{
			case i_eCFG_KF_SECTION:
#if defined CCFG_PARSER_DEBUG
	printf ("\t# PreContext Section\n");
#endif
				i_CfgParserHandleSection (p_ParserData,
					*pn_CurrentContext);
				break;
			case i_eCFG_KF_VAR:
#if defined CCFG_PARSER_DEBUG
	printf ("PreContext Variable\n");
#endif
				i_CfgParserHandleVariable (p_ParserData,
					*pn_CurrentContext);
				break;
			case i_eCFG_KF_PREDEF:
#if defined CCFG_PARSER_DEBUG
	printf ("\t# PreContext PreDef\n");
#endif
				i_CfgParserHandleVariable (p_ParserData,
					*pn_CurrentContext);
				break;
			default:
#if defined CCFG_PARSER_DEBUG
	printf ("\t# default\n");
#endif
				/*
				 *  don't handle errors, to support
				 *  future extensions.
				 */
				break;
		} /* switch */
	} /* if */
	if (pn_PreContext != 0)
	{	/* end of section ? */
		if ((*pn_CurrentContext == i_eCFG_KF_SECTION)
		&&  (*pn_PreContext == i_eCFG_KF_SECTION))
		{
#if defined CCFG_PARSER_DEBUG
	printf ("\t# End of Section -> parent\n");
#endif
			CfgParentSection (p_ParserData->p_Config);
		}
	}
	p_ParserData->n_LastAction = i_eCFG_LA_ENDTAG;
	free (p_ParserData->psz_CurrentString);
} /* i_CfgXMLEndTagHandler() */

/********************************************************************/
/** internal function
 *
 *  this is the XML data handler.
 *  called by the parser of the expat library. the parameters are
 *  given by the expat library. because we can not change them, but
 *  have to manipulte a 't_config' object, we have a global variable
 *  called 'p_Config'. to eliminate all global variables we have to 
 *  integrate expat to Cfg and manipulate all expat-functions
 *  accordingly.
 *
 *  defined in 'cfgparser.c'
 *
 */
/********************************************************************/
static void
i_CfgXMLDataHandler
	(
	void*		UserData,
	const XML_Char*	Content,
	int		Len
	)
{
	char*		psz_Str = 0;
	int*		pn_CurrentContext;
	t_ParserData*	p_ParserData;

#if defined CCFG_PARSER_DEBUG
	printf ("i_CfgXMLDataHandler() start\n");
	if (p_ParserData->psz_CurrentString != 0)
		printf ("\t# current string: '%s'\n",
		  p_ParserData->psz_CurrentString);
	else	printf ("\t# current string empty!\n");
#endif
#if 0
	if (Len < 2)
	{ 
#if defined CCFG_PARSER_DEBUG
		printf ("psz_Str len < 2!\n");
#endif
		return;
	}
#endif
	p_ParserData = (t_ParserData*) UserData;
	i_CfgStrNCpy (&psz_Str, (char*) Content, Len);
	/*
	 * 	XML does not allow trimming of strings
	 */
#if 0
	psz_Str = i_CfgTrimString (psz_Str);
#endif
	if ((psz_Str == 0) || (strlen (psz_Str) == 0))
	{
#if defined CCFG_PARSER_DEBUG
		printf ("psz_Str is 0!\n");
#endif
		return;
	}
	if (!CfgStack_Empty (p_ParserData->p_ParserStack))
	{
		pn_CurrentContext = ((int*) CfgStack_Top (
			p_ParserData->p_ParserStack));
	}
	if (p_ParserData->n_LastAction == i_eCFG_LA_DATA)
	{	/*
		 * the last action was to assign a string to
		 * 'psz_CurrentString'. if we are here again, the 
		 * string goes over multiple lines. so we add
		 * a linefeed here
		 */
#if 0
		i_CfgStrCat (&(p_ParserData->psz_CurrentString), "\n");
#endif
		i_CfgStrCat (&(p_ParserData->psz_CurrentString),
			psz_Str);
#if defined CCFG_PARSER_DEBUG
	if (p_ParserData->psz_CurrentString != 0)
		printf ("\t# current string: '%s'\n",
		  p_ParserData->psz_CurrentString);
	else	printf ("\t# current string empty!\n");
#endif
	}
	else
	{
		p_ParserData->psz_CurrentString = psz_Str;
	}
#if defined CCFG_PARSER_DEBUG
	if (p_ParserData->psz_CurrentString != 0)
		printf ("\t# current string: '%s'\n",
		  p_ParserData->psz_CurrentString);
	else	printf ("\t# current string empty!\n");
#endif
	p_ParserData->n_LastAction = i_eCFG_LA_DATA;
} /* i_CfgXMLDataHandler() */

/********************************************************************/
/** internal function
 *
 *  read a config file and parse it into a t_config*
 *
 *  defined in 'cfgparser.c'
 *
 */
/********************************************************************/
void
i_CfgParseConfig
	(
	t_config**	pp_Cfg,
	FILE*		CfgFile
	)
{
	char		sz_Buffer[BUFSIZ];
	XML_Parser	Parser;
	int		n_Done;
	size_t		l_Len;
	t_ParserData*	p_ParserData;

#if defined CCFG_PARSER_DEBUG
	printf ("i_CfgParseConfig() start\n");
#endif
	if (!CfgFile)
	{
		return;
	}
	p_ParserData = (t_ParserData*) malloc (sizeof (t_ParserData));
	p_ParserData->p_ParserStack = CfgStack_Create ();
	Parser = XML_ParserCreate (NULL);
	XML_SetUserData (Parser, p_ParserData);
	XML_SetElementHandler (Parser, i_CfgXMLStartTagHandler,
		i_CfgXMLEndTagHandler);
	XML_SetCharacterDataHandler (Parser, i_CfgXMLDataHandler);
	p_ParserData->p_Config = CfgCreate ();
	p_ParserData->n_LastAction = i_eCFG_LA_NOTHING;
	do
	{
		l_Len = fread (sz_Buffer, 1, sizeof (sz_Buffer),
			CfgFile);
		n_Done = (l_Len < sizeof (sz_Buffer));
		if (!XML_Parse (Parser, sz_Buffer, l_Len, n_Done))
		{
			fprintf(stderr, "%s at line %d\n",
				XML_ErrorString(
					XML_GetErrorCode(Parser)),
				XML_GetCurrentLineNumber(Parser));
			return;
		}
	} while (!n_Done);
	XML_ParserFree (Parser);
	CfgStack_Destroy (p_ParserData->p_ParserStack);
	(*pp_Cfg) = p_ParserData->p_Config;
	free (p_ParserData);
#if defined CCFG_PARSER_DEBUG
	printf ("i_CfgParseConfig() stop\n");
#endif
} /* CfgParseConfig() */
